"""
    This module provides the framework for making multiobjective evolutionary computations.
    
    .. Copyright (C) 2009  Inspired Intelligence Initiative

    .. This program is free software: you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation, either version 3 of the License, or
       (at your option) any later version.

    .. This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

    .. You should have received a copy of the GNU General Public License
       along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import math
from ecspy import ec
from ecspy import archivers
from ecspy import selectors
from ecspy import replacers
from ecspy import terminators
from ecspy import variators


class Pareto(object):
    """Represents a Pareto multiobjective solution.
    
    A Pareto solution is a multiobjective value that can be compared
    to other Pareto values using Pareto preference. This means that
    a solution dominates, or is better than, another solution if it is
    better than or equal to the other solution in all objectives and
    strictly better in at least one objective.
    
    Since some problems may mix maximization and minimization among
    different objectives, an optional `maximize` parameter may be
    passed upon construction of the Pareto object. This parameter
    may be a list of Booleans of the same length as the set of 
    objective values. If this parameter is used, then the `maximize`
    parameter of the evolutionary computation's `evolve` method 
    should be left as the default True value in order to avoid
    confusion. (Setting the `evolve`'s parameter to False would
    essentially invert all of the Booleans in the Pareto `maximize`
    list.) So, if all objectives are of the same type (either
    maximization or minimization), then it is best simply to use
    the `maximize` parameter of the `evolve` method and to leave
    the `maximize` parameter of the Pareto initialization set to
    its default True value. However, if the objectives are mixed
    maximization and minimization, it is best to leave the `evolve`'s
    `maximize` parameter set to its default True value and specify
    the Pareto's `maximize` list to the appropriate Booleans.
    
    """
    def __init__(self, values=[], maximize=True):
        self.values = values
        try:
            iter(maximize)
        except TypeError:
            maximize = [maximize for v in values]
        self.maximize = maximize
        
    def __len__(self):
        return len(self.values)
    
    def __getitem__(self, key):
        return self.values[key]
        
    def __iter__(self):
        return iter(self.values)
    
    def __lt__(self, other):
        if len(self.values) != len(other.values):
            raise NotImplementedError
        else:
            not_worse = True
            strictly_better = False
            for x, y, m in zip(self.values, other.values, self.maximize):
                if m:
                    if x > y:
                        not_worse = False
                    elif y > x:
                        strictly_better = True
                else:
                    if x < y:
                        not_worse = False
                    elif y < x:
                        strictly_better = True
            return not_worse and strictly_better
            
    def __le__(self, other):
        return self < other or not other < self
        
    def __gt__(self, other):
        return other < self
        
    def __ge__(self, other):
        return other < self or not self < other
        
    def __eq__(self, other):
        return self.values == other.values
    
    def __str__(self):
        return str(self.values)
        
    def __repr__(self):
        return str(self.values)


class NSGA2(ec.EvolutionaryComputation):
    """Evolutionary computation representing the nondominated sorting genetic algorithm.
    
    This class represents the nondominated sorting genetic algorithm (NSGA-II)
    of Kalyanmoy Deb et al. It uses nondominated sorting with crowding for 
    replacement, binary tournament selection to produce *population size*
    children, and a Pareto archival strategy. The remaining operators take 
    on the typical default values but they may be specified by the designer.
    
    """
    def __init__(self, random):
        ec.EvolutionaryComputation.__init__(self, random)
        self.archiver = archivers.best_archiver
        self.replacer = replacers.nsga_replacement
        self.selector = selectors.tournament_selection
    
    def evolve(self, generator, evaluator, pop_size=100, seeds=[], maximize=True, bounder=ec.Bounder(), **args):
        args.setdefault('num_selected', pop_size)
        args.setdefault('tourn_size', 2)
        return ec.EvolutionaryComputation.evolve(self, generator, evaluator, pop_size, seeds, maximize, bounder, **args)

    
class PAES(ec.EvolutionaryComputation):
    """Evolutionary computation representing the Pareto Archived Evolution Strategy.
    
    This class represents the Pareto Archived Evolution Strategy of Joshua
    Knowles and David Corne. It is essentially a (1+1)-ES with an adaptive
    grid archive that is used as a part of the replacement process. 
    
    """
    def __init__(self, random):
        ec.EvolutionaryComputation.__init__(self, random)
        self.archiver = archivers.adaptive_grid_archiver
        self.selector = selectors.default_selection
        self.variator = variators.gaussian_mutation
        self.replacer = replacers.paes_replacement  

    def evolve(self, generator, evaluator, pop_size=1, seeds=[], maximize=True, bounder=ec.Bounder(), **args):
        final_pop = ec.EvolutionaryComputation.evolve(self, generator, evaluator, pop_size, seeds, maximize, bounder, **args)
        try:
            del self.archiver.grid_population
        except AttributeError:
            pass
        try:
            del self.archiver.global_smallest
        except AttributeError:
            pass
        try:
            del self.archiver.global_largest
        except AttributeError:
            pass
        return final_pop
    




import random
from ecspy import ec
from ecspy import variators
from ecspy import replacers
from ecspy import terminators
from ecspy import observers

def my_constraint_function(candidate):
    """Return the number of constraints that candidate violates."""
    # In this case, we'll just say that the point has to lie 
    # within a circle of radius 1.
    if candidate[0]**2 + candidate[1]**2 > 1:
        return 1
    else:
        return 0

def my_generator(random, args):
    # Create pairs in the range [-2, 2].
    return [random.uniform(-2.0, 2.0) for i in range(2)]

def my_evaluator(candidates, args):
    # The fitness will be how close the point is to
    # the edge of the circle. (We're maximizing, in
    # this case.)
    WORST_FIT = -1
    fitness = []
    for cand in candidates:
        if my_constraint_function(cand) > 0:
            fitness.append(WORST_FIT)
        else:
            fitness.append(cand[0]**2 + cand[1]**2)
    return fitness

def constrained_tournament_selection(random, population, args):
    num_selected = args.setdefault('num_selected', 1)
    constraint_func = args.setdefault('constraint_func', None)
    tourn_size = 2
    pop = list(population)
    selected = []
    for _ in range(num_selected):
        tourn = random.sample(pop, tourn_size)
        # If there is not a constraint function,
        # just do regular tournament selection.
        if constraint_func is None:
            selected.append(max(tourn))
        else:
            cons = [constraint_func(t.candidate) for t in tourn]
            # If no constraints are violated, just do 
            # regular tournament selection.
            if max(cons) == 0:
                selected.append(max(tourn))
            # Otherwise, choose the least violator 
            # (which may be a non-violator).
            else:
                selected.append(tourn[cons.index(min(cons))])
    return selected

r = random.Random()
myec = ec.EvolutionaryComputation(r)
myec.selector = constrained_tournament_selection
myec.variator = variators.gaussian_mutation
myec.replacer = replacers.generational_replacement
myec.terminator = terminators.evaluation_termination
myec.observer = observers.screen_observer
pop = myec.evolve(my_generator, my_evaluator, 
                  pop_size=100, 
                  bounder=ec.Bounder([-2.0] * 2, [2.0] * 2),
                  num_selected=100,
                  constraint_func=my_constraint_function, 
                  mutation_rate=0.5,
                  max_evaluations=2000)
                  
import pylab
x = []
y = []
c = []
pop.sort()
num_feasible = len([p for p in pop if p.fitness >= 0])
feasible_count = 0
for i, p in enumerate(pop):
    x.append(p.candidate[0])
    y.append(p.candidate[1])
    if i == len(pop) - 1:
        c.append('r')
    elif p.fitness < 0:
        c.append('0.98')
    else:
        c.append(str(1 - feasible_count / float(num_feasible)))
        feasible_count += 1
angles = pylab.linspace(0, 2*pylab.pi, 100)
pylab.plot(pylab.cos(angles), pylab.sin(angles), color='b')
pylab.scatter(x, y, color=c)
pylab.savefig('constraint_example.pdf', format='pdf')

                  



"""
    This module provides the framework for making evolutionary computations.
    
    .. Copyright (C) 2009  Inspired Intelligence Initiative

    .. This program is free software: you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation, either version 3 of the License, or
       (at your option) any later version.

    .. This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

    .. You should have received a copy of the GNU General Public License
       along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import time
import copy
import logging
import itertools
from ecspy import selectors
from ecspy import variators
from ecspy import replacers
from ecspy import migrators
from ecspy import archivers
from ecspy import terminators
from ecspy import observers


class Bounder(object):
    """Defines a basic bounding function for numeric lists.
    
    This callable class acts as a function that bounds a 
    numeric list between the lower and upper bounds specified.
    These bounds can be single values or lists of values. For
    instance, if the candidate is composed of five values, each
    of which should be bounded between 0 and 1, you can say
    ``Bounder([0, 0, 0, 0, 0], [1, 1, 1, 1, 1])`` or just
    ``Bounder(0, 1)``. If either the ``lower_bound`` or 
    ``upper_bound`` argument is ``None``, the Bounder leaves 
    the candidate unchanged (which is the default behavior).
    
    A bounding function is necessary to ensure that all 
    evolutionary operators respect the legal bounds for 
    candidates. If the user is using only custom operators
    (which would be aware of the problem constraints), then 
    those can obviously be tailored to enforce the bounds
    on the candidates themselves. But the built-in operators
    make only minimal assumptions about the candidate solutions.
    Therefore, they must rely on an external bounding function
    that can be user-specified (so as to contain problem-specific
    information). As a historical note, ECsPy was originally 
    designed to require the maximum and minimum values for all
    components of the candidate solution to be passed to the
    ``evolve`` method. However, this was replaced by the bounding
    function approach because it made fewer assumptions about
    the structure of a candidate (e.g., that candidates were 
    going to be lists) and because it allowed the user the
    flexibility to provide more elaborate boundings if needed.
    
    In general, a user-specified bounding function must accept
    two arguments: the candidate to be bounded and the keyword
    argument dictionary. Typically, the signature of such a 
    function would be ``bounding_function(candidate, args)``.
    This function should return the resulting candidate after 
    bounding has been performed.
    
    Public Attributes:
    
    - *lower_bound* -- the lower bound for a candidate
    - *upper_bound* -- the upper bound for a candidate
    
    """
    def __init__(self, lower_bound=None, upper_bound=None):
        self.lower_bound = lower_bound
        self.upper_bound = upper_bound
        if self.lower_bound is not None and self.upper_bound is not None:
            try:
                iter(self.lower_bound)
            except TypeError:
                self.lower_bound = itertools.repeat(self.lower_bound)
            try:
                iter(self.upper_bound)
            except TypeError:
                self.upper_bound = itertools.repeat(self.upper_bound)
            

    def __call__(self, candidate, args):
        # The default would be to leave the candidate alone
        # unless both bounds are specified.
        if self.lower_bound is None or self.upper_bound is None:
            return candidate
        else:
            try:
                iter(self.lower_bound)
            except TypeError:
                self.lower_bound = [self.lower_bound] * len(candidate)
            try:
                iter(self.upper_bound)
            except TypeError:
                self.upper_bound = [self.upper_bound] * len(candidate)
            bounded_candidate = copy.copy(candidate)
            for i, (c, lo, hi) in enumerate(zip(candidate, self.lower_bound, self.upper_bound)):
                bounded_candidate[i] = max(min(c, hi), lo)
            return bounded_candidate


class Individual(object):
    """Represents an individual in an evolutionary computation.
    
    An individual is defined by its candidate solution and the
    fitness (or value) of that candidate solution.
    
    Public Attributes:
    
    - *candidate* -- the candidate solution
    - *fitness* -- the value of the candidate solution
    - *birthdate* -- the system time at which the individual was created
    - *maximize* -- Boolean value stating use of maximization
    
    """
    def __init__(self, candidate=None, maximize=True):
        self.candidate = candidate
        self.fitness = None
        self.birthdate = time.time()
        self.maximize = maximize
    
    def __setattr__(self, name, val):
        if name == 'candidate':
            self.__dict__[name] = val
            self.fitness = None
        else:
            self.__dict__[name] = val
    
    def __str__(self):
        return '%s : %s' % (str(self.candidate), str(self.fitness))
        
    def __repr__(self):
        return '<Individual: candidate = %s, fitness = %s, birthdate = %s>' % ( str(self.candidate), str(self.fitness), self.birthdate )
        
    def __lt__(self, other):
        if self.fitness is not None and other.fitness is not None:
            if self.maximize: 
                return self.fitness < other.fitness
            else:
                return self.fitness > other.fitness
        else:
            raise Exception('fitness is not defined')

    def __le__(self, other):
        return self < other or not other < self
            
    def __gt__(self, other):
        if self.fitness is not None and other.fitness is not None:
            return other < self
        else:
            raise Exception('fitness is not defined')

    def __ge__(self, other):
        return other < self or not self < other
        
    def __lshift__(self, other):
        return self < other
    
    def __rshift__(self, other):
        return other < self
        
    def __ilshift__(self, other):
        raise TypeError("unsupported operand type(s) for <<=: 'Individual' and 'Individual'")
    
    def __irshift__(self, other):
        raise TypeError("unsupported operand type(s) for >>=: 'Individual' and 'Individual'")
        
    def __eq__(self, other):
        return self.candidate == other.candidate
        
    def __ne__(self, other):
        return self.candidate != other.candidate


class EvolutionExit(Exception):
    """An exception that may be raised and caught to end the evolution.
    
    This is an empty exception class that can be raised by the user
    at any point in the code and caught outside of the ``evolve``
    method. 
    
    """
    pass


class EvolutionaryComputation(object):
    """Represents a basic evolutionary computation.
    
    This class encapsulates the components of a generic evolutionary
    computation. These components are the selection mechanism, the
    variation operators, the replacement mechanism, the migration
    scheme, and the observers.
    
    Public Attributes:
    
    - *selector* -- the selection operator
    - *variator* -- the (possibly list of) variation operator(s)
    - *replacer* -- the replacement operator
    - *migrator* -- the migration operator
    - *archiver* -- the archival operator
    - *observer* -- the (possibly list of) observer(s)
    - *terminator* -- the (possibly list of) terminator(s)
    
    The following public attributes do not have legitimate values
    until after the ``evolve`` method executes:
    
    - *termination_cause* -- the name of the function causing 
      ``evolve`` to terminate
    - *generator* -- the generator function passed to ``evolve``
    - *evaluator* -- the evaluator function passed to ``evolve``
    - *bounder* -- the bounding function passed to ``evolve``
    - *maximize* -- Boolean stating use of maximization passed to ``evolve``
    - *archive* -- the archive of individuals
    - *population* -- the population of individuals
    - *num_evaluations* -- the number of fitness evaluations used
    - *num_generations* -- the number of generations processed
    - *logger* -- the logger to use (defaults to the logger 'ecspy.ec')
    
    Note that the attributes above are, in general, not intended to 
    be modified by the user. (They are intended for the user to query
    during or after the ``evolve`` method's execution.) However, 
    there may be instances where it is necessary to modify them 
    within other functions. This is possible but should be the 
    exception, rather than the rule.
    
    If logging is desired, the following basic code segment can be 
    used in the ``main`` or calling scope to accomplish that::
    
        logger = logging.getLogger('ecspy.ec')
        logger.setLevel(logging.DEBUG)
        file_handler = logging.FileHandler('ec.log', mode='w')
        file_handler.setLevel(logging.DEBUG)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        file_handler.setFormatter(formatter)
        logger.addHandler(file_handler)
    
    Protected Attributes:
    
    - *_random* -- the random number generator object
    - *_kwargs* -- the dictionary of keyword arguments initialized
      from the *args* parameter in the *evolve* method
    
    Public Methods:
    
    - ``evolve`` -- performs the evolution and returns the final
      archive of individuals
    
    """
    def __init__(self, random):
        self.selector = selectors.default_selection
        self.variator = variators.default_variation
        self.replacer = replacers.default_replacement
        self.migrator = migrators.default_migration
        self.observer = observers.default_observer
        self.archiver = archivers.default_archiver
        self.terminator = terminators.default_termination
        self.termination_cause = None
        self.generator = None
        self.evaluator = None
        self.bounder = None
        self.maximize = True
        self.archive = None
        self.population = None
        self.num_evaluations = 0
        self.num_generations = 0
        self.logger = logging.getLogger('ecspy.ec')
        self._random = random
        self._kwargs = dict()
        
    def _should_terminate(self, pop, ng, ne):
        terminate = False
        fname = ''
        try:
            for clause in self.terminator:
                self.logger.debug('termination test using %s at generation %d and evaluation %d' % (clause.__name__, ng, ne))
                terminate = terminate or clause(population=pop, num_generations=ng, num_evaluations=ne, args=self._kwargs)
                if terminate:
                    fname = clause.__name__
                    break
        except TypeError:
            self.logger.debug('termination test using %s at generation %d and evaluation %d' % (self.terminator.__name__, ng, ne))
            terminate = self.terminator(population=pop, num_generations=ng, num_evaluations=ne, args=self._kwargs)
            fname = self.terminator.__name__
        if terminate:
            self.termination_cause = fname
            self.logger.debug('termination from %s at generation %d and evaluation %d' % (self.termination_cause, ng, ne))
        return terminate
        
    
    def evolve(self, generator, evaluator, pop_size=100, seeds=[], maximize=True, bounder=Bounder(), **args):
        """Perform the evolution.
        
        This function creates a population and then runs it through a series
        of evolutionary epochs until the terminator is satisfied. The general
        outline of an epoch is selection, variation, evaluation, replacement,
        migration, archival, and observation. The function returns a list of
        elements of type ``Individual`` representing the individuals contained
        in the final population.
        
        Arguments:
        
        - *generator* -- the function to be used to generate candidate solutions 
        - *evaluator* -- the function to be used to evaluate candidate solutions
        - *pop_size* -- the number of Individuals in the population (default 100)
        - *seeds* -- an iterable collection of candidate solutions to include
          in the initial population (default [])
        - *maximize* -- Boolean value stating use of maximization (default True)
        - *bounder* -- a function used to bound candidate solutions (default Bounder())
        - *args* -- a dictionary of keyword arguments

        Note that the *_kwargs* class variable will be initialized to the args 
        parameter here. It will also be modified to include the following 'built-in' 
        keyword arguments:
        
        - *_ec* -- the evolutionary computation (this object)
        
        """
        self._kwargs = args
        self._kwargs['_ec'] = self
        
        self.termination_cause = None
        self.generator = generator
        self.evaluator = evaluator
        self.bounder = bounder
        self.maximize = maximize
        self.population = []
        self.archive = []
        
        # Create the initial population.
        try:
            iter(seeds)
        except TypeError:
            seeds = [seeds]
        initial_cs = list(seeds)
        num_generated = max(pop_size - len(seeds), 0)
        i = 0
        self.logger.debug('generating initial population')
        while i < num_generated:
            cs = generator(random=self._random, args=self._kwargs)
            if cs not in initial_cs:
                initial_cs.append(cs)
                i += 1
        self.logger.debug('evaluating initial population')
        initial_fit = evaluator(candidates=initial_cs, args=self._kwargs)
        
        for cs, fit in zip(initial_cs, initial_fit):
            ind = Individual(cs, maximize=maximize)
            ind.fitness = fit
            self.population.append(ind)
        self.logger.debug('population size is now %d' % len(self.population))
        
        self.num_evaluations = len(initial_fit)
        self.num_generations = 0
        
        self.logger.debug('archiving initial population')
        self.archive = self.archiver(random=self._random, population=list(self.population), archive=list(self.archive), args=self._kwargs)
        self.logger.debug('archive size is now %d' % len(self.archive))
        self.logger.debug('population size is now %d' % len(self.population))
                
        if isinstance(self.observer, (list, tuple)):
            for obs in self.observer:
                self.logger.debug('observation using %s at generation %d and evaluation %d' % (obs.__name__, self.num_generations, self.num_evaluations))
                obs(population=list(self.population), num_generations=self.num_generations, num_evaluations=self.num_evaluations, args=self._kwargs)
        else:
            self.logger.debug('observation using %s at generation %d and evaluation %d' % (self.observer.__name__, self.num_generations, self.num_evaluations))
            self.observer(population=list(self.population), num_generations=self.num_generations, num_evaluations=self.num_evaluations, args=self._kwargs)
        
        while not self._should_terminate(list(self.population), self.num_generations, self.num_evaluations):
            # Select individuals.
            self.logger.debug('selection using %s at generation %d and evaluation %d' % (self.selector.__name__, self.num_generations, self.num_evaluations))
            parents = self.selector(random=self._random, population=list(self.population), args=self._kwargs)
            self.logger.debug('selected %d candidates' % len(parents))
            parent_cs = [copy.deepcopy(i.candidate) for i in parents]
            offspring_cs = parent_cs
            
            if isinstance(self.variator, (list, tuple)):
                for op in self.variator:
                    self.logger.debug('variation using %s at generation %d and evaluation %d' % (op.__name__, self.num_generations, self.num_evaluations))
                    offspring_cs = op(random=self._random, candidates=offspring_cs, args=self._kwargs)
            else:
                self.logger.debug('variation using %s at generation %d and evaluation %d' % (self.variator.__name__, self.num_generations, self.num_evaluations))
                offspring_cs = self.variator(random=self._random, candidates=offspring_cs, args=self._kwargs)
            self.logger.debug('created %d offspring' % len(offspring_cs))
            
            # Evaluate offspring.
            self.logger.debug('evaluation using %s at generation %d and evaluation %d' % (evaluator.__name__, self.num_generations, self.num_evaluations))
            offspring_fit = evaluator(candidates=offspring_cs, args=self._kwargs)
            offspring = []
            for cs, fit in zip(offspring_cs, offspring_fit):
                off = Individual(cs, maximize=maximize)
                off.fitness = fit
                offspring.append(off)
            self.num_evaluations += len(offspring_fit)        

            # Replace individuals.
            self.logger.debug('replacement using %s at generation %d and evaluation %d' % (self.replacer.__name__, self.num_generations, self.num_evaluations))
            self.population = self.replacer(random=self._random, population=list(self.population), parents=parents, offspring=offspring, args=self._kwargs)
            self.logger.debug('population size is now %d' % len(self.population))
            
            # Migrate individuals.
            self.logger.debug('migration using %s at generation %d and evaluation %d' % (self.migrator.__name__, self.num_generations, self.num_evaluations))
            self.population = self.migrator(random=self._random, population=list(self.population), args=self._kwargs)
            self.logger.debug('population size is now %d' % len(self.population))
            
            # Archive individuals.
            self.logger.debug('archival using %s at generation %d and evaluation %d' % (self.archiver.__name__, self.num_generations, self.num_evaluations))
            self.archive = self.archiver(random=self._random, archive=list(self.archive), population=list(self.population), args=self._kwargs)
            self.logger.debug('archive size is now %d' % len(self.archive))
            self.logger.debug('population size is now %d' % len(self.population))
            
            self.num_generations += 1
            if isinstance(self.observer, (list, tuple)):
                for obs in self.observer:
                    self.logger.debug('observation using %s at generation %d and evaluation %d' % (obs.__name__, self.num_generations, self.num_evaluations))
                    obs(population=list(self.population), num_generations=self.num_generations, num_evaluations=self.num_evaluations, args=self._kwargs)
            else:
                self.logger.debug('observation using %s at generation %d and evaluation %d' % (self.observer.__name__, self.num_generations, self.num_evaluations))
                self.observer(population=list(self.population), num_generations=self.num_generations, num_evaluations=self.num_evaluations, args=self._kwargs)
        return self.population
        

class GA(EvolutionaryComputation):
    """Evolutionary computation representing a canonical genetic algorithm.
    
    This class represents a genetic algorithm which uses, by 
    default, rank selection, n-point crossover, bit-flip mutation, 
    and generational replacement. In the case of bit-flip mutation, 
    it is expected that the candidate solution is an iterable object 
    of binary values. 
    
    Optional keyword arguments in ``evolve`` args parameter:
    
    - *num_selected* -- the number of individuals to be selected (default 1)
    - *crossover_rate* -- the rate at which crossover is performed 
      (default 1.0)
    - *num_crossover_points* -- the n crossover points used (default 1)
    - *mutation_rate* -- the rate at which mutation is performed (default 0.1)
    - *num_elites* -- number of elites to consider (default 0)
    
    """
    def __init__(self, random):
        EvolutionaryComputation.__init__(self, random)
        self.selector = selectors.rank_selection 
        self.variator = [variators.n_point_crossover, variators.bit_flip_mutation]
        self.replacer = replacers.generational_replacement
        
    def evolve(self, generator, evaluator, pop_size=100, seeds=[], maximize=True, bounder=Bounder(), **args):
        args.setdefault('num_selected', pop_size)
        return EvolutionaryComputation.evolve(self, generator, evaluator, pop_size, seeds, maximize, bounder, **args)


class ES(EvolutionaryComputation):
    """Evolutionary computation representing a canonical evolution strategy.
    
    This class represents an evolution strategy which uses, by 
    default, the default selection (i.e., all individuals are selected), 
    Gaussian mutation, and 'plus' replacement. It is assumed that the
    candidate solution is an iterable object of real values. 

    Optional keyword arguments in ``evolve`` args parameter:
    
    - *mutation_rate* -- the rate at which mutation is performed (default 0.1)
    - *mean* -- the mean used in the Gaussian function (default 0)
    - *stdev* -- the standard deviation used in the Gaussian function
      (default 1.0)
    - *use_one_fifth_rule* -- whether the 1/5 rule should be used (default False)
    
    """
    def __init__(self, random):
        EvolutionaryComputation.__init__(self, random)
        self.selector = selectors.default_selection
        self.variator = variators.gaussian_mutation
        self.replacer = replacers.plus_replacement
        

class EDA(EvolutionaryComputation):
    """Evolutionary computation representing a canonical estimation of distribution algorithm.
    
    This class represents an estimation of distribution algorithm which
    uses, by default, truncation selection, estimation of distribution 
    variation, and generational replacement. It is assumed that the
    candidate solution is an iterable object of real values. 

    Optional keyword arguments in ``evolve`` args parameter:
    
    - *num_selected* -- the number of individuals to be selected 
      (default len(population))
    - *num_offspring* -- the number of offspring to create (default 1)
    - *num_elites* -- number of elites to consider (default 0)
    
    """
    def __init__(self, random):
        EvolutionaryComputation.__init__(self, random)
        self.selector = selectors.truncation_selection
        self.variator = variators.estimation_of_distribution_variation
        self.replacer = replacers.generational_replacement
        
    def evolve(self, generator, evaluator, pop_size=100, seeds=[], maximize=True, bounder=Bounder(), **args):
        args.setdefault('num_selected', pop_size // 2)
        args.setdefault('num_offspring', pop_size)
        return EvolutionaryComputation.evolve(self, generator, evaluator, pop_size, seeds, maximize, bounder, **args)


class DEA(EvolutionaryComputation):
    """Evolutionary computation representing a differential evolutionary algorithm.
    
    This class represents a differential evolutionary algorithm which uses, by 
    default, tournament selection, differential crossover, Gaussian mutation,
    and steady-state replacement. It is expected that the candidate solution 
    is an iterable object of real values. 
    
    Optional keyword arguments in ``evolve`` args parameter:
    
    - *num_selected* -- the number of individuals to be selected (default 1)
    - *tourn_size* -- the tournament size (default 2)
    - *crossover_rate* -- the rate at which crossover is performed 
      (default 1.0)
    - *differential_phi* -- the amount of random change in the crossover 
      (default 0.1)
    - *mutation_rate* -- the rate at which mutation is performed (default 0.1)
    - *mean* -- the mean used in the Gaussian function (default 0)
    - *stdev* -- the standard deviation used in the Gaussian function
      (default 1.0)

    """
    def __init__(self, random):
        EvolutionaryComputation.__init__(self, random)
        self.selector = selectors.tournament_selection
        self.variator = [variators.differential_crossover, variators.gaussian_mutation]
        self.replacer = replacers.steady_state_replacement
        
    def evolve(self, generator, evaluator, pop_size=100, seeds=[], maximize=True, bounder=Bounder(), **args):
        args.setdefault('num_selected', 2)
        return EvolutionaryComputation.evolve(self, generator, evaluator, pop_size, seeds, maximize, bounder, **args)


class SA(EvolutionaryComputation):
    """Evolutionary computation representing simulated annealing."""
    def __init__(self, random):
        EvolutionaryComputation.__init__(self, random)
        self.selector = selectors.default_selection
        self.variator = variators.gaussian_mutation
        self.replacer = replacers.simulated_annealing_replacement
    
    def evolve(self, generator, evaluator, pop_size=1, seeds=[], maximize=True, bounder=Bounder(), **args):
        pop_size=1
        return EvolutionaryComputation.evolve(self, generator, evaluator, pop_size, seeds, maximize, bounder, **args)



"""
    This module provides pre-defined selectors for evolutionary computations.

    All selector functions have the following arguments:
    
    - *random* -- the random number generator object
    - *population* -- the population of individuals
    - *args* -- a dictionary of keyword arguments
    
    Each selector function returns the list of selected individuals.
    
    .. Copyright (C) 2009  Inspired Intelligence Initiative

    .. This program is free software: you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation, either version 3 of the License, or
       (at your option) any later version.

    .. This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

    .. You should have received a copy of the GNU General Public License
       along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


def default_selection(random, population, args):
    """Return the population.
    
    This function acts as a default selection scheme for an EC.
    It simply returns the entire population as having been 
    selected.
    
    .. Arguments:
       random -- the random number generator object
       population -- the population of individuals
       args -- a dictionary of keyword arguments
    
    """
    return population


def truncation_selection(random, population, args):
    """Selects the best individuals from the population.
    
    This function performs truncation selection, which means that only
    the best individuals from the current population are selected. This
    is a completely deterministic selection mechanism.
    
    .. Arguments:
       random -- the random number generator object
       population -- the population of individuals
       args -- a dictionary of keyword arguments

    Optional keyword arguments in args:
    
    *num_selected* -- the number of individuals to be selected 
    (default len(population))
    
    """
    num_selected = args.setdefault('num_selected', len(population))
    pool = list(population)
    pool.sort(reverse=True)
    return pool[:num_selected]

    
def uniform_selection(random, population, args):
    """Return a uniform sampling of individuals from the population.
    
    This function performs uniform selection by randomly choosing
    members of the population with replacement.
    
    .. Arguments:
       random -- the random number generator object
       population -- the population of individuals
       args -- a dictionary of keyword arguments

    Optional keyword arguments in args:
    
    *num_selected* -- the number of individuals to be selected 
    (default 1)
    
    """
    num_selected = args.setdefault('num_selected', 1)
    pop = list(population)
    selected = []
    for _ in range(num_selected):
        selected.append(pop[random.randint(0, len(pop)-1)])
    return selected


def fitness_proportionate_selection(random, population, args):
    """Return fitness proportionate sampling of individuals from the population.

    .. Arguments:
       random -- the random number generator object
       population -- the population of individuals
       args -- a dictionary of keyword arguments

    Optional keyword arguments in args:
    
    *num_selected* -- the number of individuals to be selected (default 1)
    
    """
    num_selected = args.setdefault('num_selected', 1)
    pop = list(population)
    len_pop = len(pop)
    psum = [i for i in range(len_pop)]
    pop_max_fit = (max(pop)).fitness
    pop_min_fit = (min(pop)).fitness
    
    # If we're actually doing minimimization,
    # fitness proportionate selection is not defined.
    if pop_max_fit < pop_min_fit:
        raise ValueError('Fitness proportionate selection is not valid for minimization.')
    
    # Set up the roulette wheel
    if pop_max_fit == pop_min_fit:
        psum = [(index + 1) / float(len_pop) for index in range(len_pop)]
    elif (pop_max_fit > 0 and pop_min_fit >= 0) or (pop_max_fit <= 0 and pop_min_fit < 0):
        pop.sort(reverse=True)
        psum[0] = pop[0].fitness
        for i in range(1, len_pop):
            psum[i] = pop[i].fitness + psum[i-1]
        for i in range(len_pop):
            psum[i] /= float(psum[len_pop-1])
            
    # Select the individuals
    selected = []
    for _ in range(num_selected):
        cutoff = random.random()
        lower = 0
        upper = len_pop - 1
        while(upper >= lower):
            mid = (lower + upper) // 2
            if psum[mid] > cutoff: 
                upper = mid - 1
            else: 
                lower = mid + 1
        lower = max(0, min(len_pop-1, lower))
        selected.append(pop[lower])
    return selected


def rank_selection(random, population, args):
    """Return a rank-based sampling of individuals from the population.
    
    .. Arguments:
       random -- the random number generator object
       population -- the population of individuals
       args -- a dictionary of keyword arguments

    Optional keyword arguments in args:
    
    *num_selected* -- the number of individuals to be selected (default 1)
    
    """
    num_selected = args.setdefault('num_selected', 1)

    # Set up the roulette wheel
    pop = list(population)
    len_pop = len(pop)
    pop.sort()
    psum = range(len_pop)
    den = (len_pop * (len_pop + 1)) / 2.0
    for i in range(len_pop):
        psum[i] = (i + 1) / den
    for i in range(1, len_pop):
        psum[i] += psum[i-1]
        
    # Select the individuals
    selected = []
    for _ in range(num_selected):
        cutoff = random.random()
        lower = 0
        upper = len_pop - 1
        while(upper >= lower):
            mid = (lower + upper) // 2
            if psum[mid] > cutoff: 
                upper = mid - 1
            else: 
                lower = mid + 1
        lower = max(0, min(len_pop-1, lower))
        selected.append(pop[lower])
    return selected


def tournament_selection(random, population, args):
    """Return a tournament sampling of individuals from the population.
    
    .. Arguments:
       random -- the random number generator object
       population -- the population of individuals
       args -- a dictionary of keyword arguments

    Optional keyword arguments in args:
    
    - *num_selected* -- the number of individuals to be selected (default 1)
    - *tourn_size* -- the tournament size (default 2)
    
    """
    num_selected = args.setdefault('num_selected', 1)
    tourn_size = args.setdefault('tourn_size', 2)
    pop = list(population)
    selected = []
    for _ in range(num_selected):
        tourn = random.sample(pop, tourn_size)
        selected.append(max(tourn))
    return selected





"""
    This module provides pre-defined terminators for evolutionary computations.
    
    All terminator functions have the following arguments:
    
    - *population* -- the population of Individuals
    - *num_generations* -- the number of elapsed generations
    - *num_evaluations* -- the number of candidate solution evaluations
    - *args* -- a dictionary of keyword arguments
    
    .. Copyright (C) 2009  Inspired Intelligence Initiative

    .. This program is free software: you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation, either version 3 of the License, or
       (at your option) any later version.

    .. This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

    .. You should have received a copy of the GNU General Public License
       along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import math, itertools, sys, time, warnings


def default_termination(population, num_generations, num_evaluations, args):
    """Return True.
    
    This function acts as a default termination criterion for an EC.
    
    .. Arguments:
       population -- the population of Individuals
       num_generations -- the number of elapsed generations
       num_evaluations -- the number of candidate solution evaluations
       args -- a dictionary of keyword arguments
    
    """
    return True
    

def diversity_termination(population, num_generations, num_evaluations, args):
    """Return True if population diversity is less than a minimum diversity.
    
    This function calculates the Euclidean distance between every pair of
    individuals in the population. It then compares the maximum of those
    distances with a specified minimum required diversity. This terminator 
    is really only well-defined for candidate solutions which are List 
    types of numeric values. 
    
    .. Arguments:
       population -- the population of Individuals
       num_generations -- the number of elapsed generations
       num_evaluations -- the number of candidate solution evaluations
       args -- a dictionary of keyword arguments
    
    Optional keyword arguments in args:
    
    *min_diversity* -- the minimum population diversity allowed (default 0.001)
    
    """
    min_diversity = args.setdefault('min_diversity', 0.001)
    cart_prod = itertools.product(population, population)
    distance = []
    for (p, q) in cart_prod:
        d = 0
        for x, y in zip(p.candidate, q.candidate):
            d += (x - y)**2
        distance.append(math.sqrt(d))
    if max(distance) < min_diversity:
        return True
    return False

    
def average_fitness_termination(population, num_generations, num_evaluations, args):
    """Return True if the population's average fitness is near its best fitness.
    
    This function calculates the average fitness of the population, as well
    as the best fitness. If the difference between those values is less 
    than a specified tolerance, the function returns True. 
    
    .. Arguments:
       population -- the population of Individuals
       num_generations -- the number of elapsed generations
       num_evaluations -- the number of candidate solution evaluations
       args -- a dictionary of keyword arguments
    
    Optional keyword arguments in args:
    
    *tolerance* -- the minimum allowable difference between average 
    and best fitness (default 0.001)
    
    """
    tolerance = args.setdefault('tolerance', 0.001)
    avg_fit = sum([x.fitness for x in population]) / float(len(population))
    best_fit = max([x.fitness for x in population])
    if (best_fit - avg_fit) < tolerance:
        return True
    return False


def evaluation_termination(population, num_generations, num_evaluations, args):
    """Return True if the number of function evaluations meets or exceeds a maximum.
    
    This function compares the number of function evaluations that have been 
    generated with a specified maximum. It returns True if the maximum is met
    or exceeded.
    
    .. Arguments:
       population -- the population of Individuals
       num_generations -- the number of elapsed generations
       num_evaluations -- the number of candidate solution evaluations
       args -- a dictionary of keyword arguments
    
    Optional keyword arguments in args:
    
    *max_evaluations* -- the maximum candidate solution evaluations (default len(population)) 
    
    """
    max_evaluations = args.setdefault('max_evaluations', len(population))
    if num_evaluations >= max_evaluations:
        return True
    return False


def generation_termination(population, num_generations, num_evaluations, args):
    """Return True if the number of generations meets or exceeds a maximum.
    
    This function compares the number of generations with a specified 
    maximum. It returns True if the maximum is met or exceeded.
    
    .. Arguments:
       population -- the population of Individuals
       num_generations -- the number of elapsed generations
       num_evaluations -- the number of candidate solution evaluations
       args -- a dictionary of keyword arguments
    
    Optional keyword arguments in args:
    
    *max_generations* -- the maximum generations (default 1) 
    
    """
    max_generations = args.setdefault('max_generations', 1)
    if num_generations >= max_generations:
        return True
    return False

    
def time_termination(population, num_generations, num_evaluations, args):
    """Return True if the elapsed time meets or exceeds a duration of time.
    
    This function compares the elapsed time with a specified maximum. 
    It returns True if the maximum is met or exceeded. If the `start_time`
    keyword argument is omitted, it defaults to `None` and will be set to
    the current system time (in seconds). If the `max_time` keyword argument
    is omitted, it will default to `None` and will immediately terminate.
    The `max_time` argument can be specified in seconds as a floating-point
    number, as minutes/seconds as a two-element tuple of floating-point
    numbers, or as hours/minutes/seconds as a three-element tuple of 
    floating-point numbers.
    
    .. Arguments:
       population -- the population of Individuals
       num_generations -- the number of elapsed generations
       num_evaluations -- the number of candidate solution evaluations
       args -- a dictionary of keyword arguments
    
    Optional keyword arguments in args:
    
    *start_time* -- the time from which to start measuring (default None)
    *max_time* -- the maximum time that should elapse (default None)
    
    """
    start_time = args.setdefault('start_time', None)
    max_time = args.setdefault('max_time', None)
    logging = args.get('_ec').logger

    if start_time is None:
        start_time = time.time()
        args['start_time'] = start_time
        logging.debug('time_termination terminator added without setting the start_time argument; setting start_time to current time')
    if max_time is None:
        logging.debug('time_termination terminator added without setting the max_time argument; terminator will immediately terminate')
    else:
        try:
            max_time = max_time[0] * 3600.0 + max_time[1] * 60.00 + max_time[2]
            args['max_time'] = max_time
        except TypeError:
            pass
        except IndexError:
            max_time = max_time[0] * 60 + max_time[1]
            args['max_time'] = max_time
    time_elapsed = time.time() - start_time
    return max_time is None or time_elapsed >= max_time


def user_termination(population, num_generations, num_evaluations, args):
    """Return True if user presses the ESC key when prompted.
    
    This function prompts the user to press the ESC key to terminate the 
    evolution. The prompt persists for a specified number of seconds before
    evolution continues. Additionally, the function can be customized to 
    allow any press of the ESC key to be stored until the next time this 
    function is called. This function makes use of the ``msvcrt`` (Windows)
    and ``curses`` (Unix) libraries. Other operating systems may not be 
    supported.
    
    .. Arguments:
       population -- the population of Individuals
       num_generations -- the number of elapsed generations
       num_evaluations -- the number of candidate solution evaluations
       args -- a dictionary of keyword arguments
    
    Optional keyword arguments in args:
    
    - *termination_response_timeout* -- the number of seconds to wait for 
      the user to press the ESC key (default 5)
    - *clear_termination_buffer* -- whether the keyboard buffer should be 
      cleared before allowing the user to press a key (default True)
    
    """
    def getch():
        unix = ('darwin', 'linux2')
        if sys.platform not in unix:
            try:
                import msvcrt
            except ImportError:
                print 'user_termination cannot import required module...\n are you running DOS or something?'
                return -1
            if msvcrt.kbhit():
                return msvcrt.getch()
            else:
                return -1

        elif sys.platform in unix:
            def _getch(stdscr):
                stdscr.nodelay(1)
                ch = stdscr.getch()
                stdscr.nodelay(0)
                return ch
            import curses
            return curses.wrapper(_getch)
    
    num_secs = args.get('termination_response_timeout', 5)
    clear_buffer = args.get('clear_termination_buffer', True)
    if clear_buffer:
        while getch() > -1:
            pass
    sys.stdout.write('Press ESC to terminate (%d secs):' % num_secs)
    count = 1
    start = time.time()
    while time.time() - start < num_secs:
        ch = getch()
        if ch > -1 and ord(ch) == 27:
            sys.stdout.write('\n\n')
            return True
        elif time.time() - start == count:
            sys.stdout.write('.')
            count += 1
    sys.stdout.write('\n')
    return False    


# Standard library
import os
import sys

# Set up Paver
import paver
import paver.doctools
import paver.misctasks
from paver.path import path
from paver.easy import *
import paver.setuputils
paver.setuputils.install_distutils_tasks()
try:
    from sphinxcontrib import paverutils
except:
    paverutils = None


PROJECT = 'ecspy'
VERSION = '1.0'

# The sphinx templates expect the VERSION in the shell environment
os.environ['VERSION'] = VERSION

# Read the long description to give to setup
README = path('README.txt').text()

# Scan the input for package information
# to grab any data files (text, images, etc.) 
# associated with sub-packages.
PACKAGE_DATA = paver.setuputils.find_package_data(PROJECT, 
                                                  package=PROJECT,
                                                  only_in_packages=False
                                                  )

options(
    setup=Bunch(
        name = PROJECT,
        version = VERSION,
        description='A framework for creating evolutionary computations in Python.',
        long_description=README,
        author='Aaron Garrett',
        author_email='aaron.lee.garrett@gmail.com',
        url='http://%s.googlecode.com' % PROJECT,
        download_url = 'http://%s.googlecode.com/files/%s-%s.zip' % (PROJECT, PROJECT, VERSION),
        license='GPLv3+',
        platforms=('Any'),
        keywords=('python', 'optimization', 'evolutionary', 'computation', 'genetic algorithm', 
                  'particle swarm', 'estimation of distribution', 'differential evolution',
                  'nsga', 'paes'),

        classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'Intended Audience :: Science/Research',
          'License :: OSI Approved :: GNU General Public License (GPL)',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 2',
          'Topic :: Scientific/Engineering :: Artificial Intelligence'
          ],
        
        packages = [PROJECT, '%s.contrib' % PROJECT, '%s.variators' % PROJECT],
        package_data=PACKAGE_DATA,
    ),
    
    sdist = Bunch(
    ),
    
    sphinx = Bunch(
        sourcedir='docs',
        docroot = '.',
        builder = 'html',
        doctrees='docs/_build/doctrees',
        confdir = 'docs',
    ),

    html = Bunch(
        builddir='docs',
        outdir='html',
        templates='pkg',
    ),

    # Some of the files include [[[ as part of a nested list data structure,
    # so change the tags cog looks for to something less likely to appear.
    cog=Bunch(
        beginspec='{{{cog',
        endspec='}}}',
        endoutput='{{{end}}}',
    ),
    
    minilib=Bunch(
        extra_files=['doctools']
    ),
)

def run_script(input_file, script_name, interpreter='python'):
    """Run a script in the context of the input_file's directory, 
    return the text output formatted to be included as an rst
    literal text block.
    """
    from paver.easy import sh
    from paver.path import path
    rundir = path(input_file).dirname()
    output_text = sh('cd %(rundir)s && %(interpreter)s %(script_name)s 2>&1' % vars(), capture=True)
    response = '\n::\n\n\t$ %(interpreter)s %(script_name)s\n\t' % vars()
    response += '\n\t'.join(output_text.splitlines())
    while not response.endswith('\n\n'):
        response += '\n'
    return response
    
    
# Stuff run_script() into the builtins so we don't have to
# import it in all of the cog blocks where we want to use it.
__builtins__['run_script'] = run_script


def remake_directories(*dirnames):
    """Remove the directories and recreate them.
    """
    for d in dirnames:
        d = path(d)
        if d.exists():
            d.rmtree()
        d.mkdir()
    return

@task
@needs(['cog'])
def html(options):
    if paverutils is None:
        raise RuntimeError('Could not find sphinxcontrib.paverutils, will not be able to build HTML output.')
    paverutils.html(options)
    return

@task
@needs(['generate_setup', 'minilib', 
        #'html_clean', 
        'setuptools.command.sdist'
        ])
def sdist(options):
    """Create a source distribution.
    """
    pass

@task
def html_clean(options):
    """Remove sphinx output directories before building the HTML.
    """
    remake_directories(options.sphinx.doctrees, options.html.outdir)
    html(options)
    return




import random
import time
import math
import itertools
from ecspy import ec
from ecspy import selectors
from ecspy import terminators
from ecspy import variators
from ecspy import replacers
from ecspy import observers


def my_distance(x, y):
    return sum([abs(a - b) for a, b in zip(x, y)])

def generate(random, args):
    return [random.uniform(0, 26)]
    

def evaluate(candidates, args):
    fitness = []
    for cand in candidates:
        fit = sum([math.sin(c) for c in cand])
        fitness.append(fit)
    return fitness


def main(do_plot=True, prng=None):
    if prng is None:
        prng = random.Random()
        prng.seed(time.time()) 
    
    ea = ec.EvolutionaryComputation(prng)
    ea.selector = selectors.tournament_selection
    ea.replacer = replacers.crowding_replacement
    ea.variator = variators.gaussian_mutation
    ea.terminator = terminators.evaluation_termination

    final_pop = ea.evolve(generate, evaluate, pop_size=20, bounder=ec.Bounder(0, 26),
                          max_evaluations=5000,
                          num_selected=20,
                          mutation_rate=1.0,
                          crowding_distance=10,
                          distance_function=my_distance)
                          
    if do_plot:
        import pylab
        x = []
        y = []
        for p in final_pop:
            x.append(p.candidate[0])
            y.append(math.sin(p.candidate[0]))
        t = [(i / 1000.0) * 26.0 for i in range(1000)]
        s = [math.sin(a) for a in t]
        pylab.plot(t, s, color='b')
        pylab.scatter(x, y, color='r')
        pylab.axis([0, 26, 0, 1.1])
        pylab.savefig('niche_example.pdf', format='pdf')
        pylab.show()
    return ea

if __name__ == '__main__':
    main()




    
    
    


# Title:  Moon probe evolution example
# Author: Mike Vella

#start_imports
import os
import math
import pylab
import itertools
from matplotlib import pyplot as plt
from matplotlib.patches import Circle
from random import Random
from time import time
from ecspy import ec
from ecspy import terminators
from ecspy import observers
from ecspy import selectors
from ecspy import replacers
from ecspy import variators
#end_imports

# All units are in SI unless stated otherwise.

# Global constants
#start_globals
G = 6.67300e-11 # Universal gravitational constant
earth_mass = 5.9742e24
earth_radius = 6.378e6
moon_mass = 7.36e22
moon_radius = 1.737e6
moon_position = (384403e3, 0)
earth_position = (0, 0)
#end_globals

def pairwise(iterable):
    """s -> (s0,s1), (s1,s2), (s2, s3), ..."""
    a, b = itertools.tee(iterable)
    next(b, None)
    return itertools.izip(a, b)
    
def distance_between(position_a, position_b):
    return math.sqrt((position_a[0] - position_b[0])**2 + (position_a[1] - position_b[1])**2)
    
def gravitational_force(position_a, mass_a, position_b, mass_b):
    """Returns the gravitational force between the two bodies a and b."""
    distance = distance_between(position_a, position_b)

    # Calculate the direction and magnitude of the force.
    angle = math.atan2(position_a[1] - position_b[1], position_a[0] - position_b[0])
    magnitude = G * mass_a * mass_b / (distance**2)

    # Find the x and y components of the force.
    # Determine sign based on which one is the larger body.
    sign = -1 if mass_b > mass_a else 1
    x_force = sign * magnitude * math.cos(angle)
    y_force = sign * magnitude * math.sin(angle)
    return x_force, y_force

def force_on_satellite(position, mass):
    """Returns the total gravitational force acting on the body from the Earth and Moon."""
    earth_grav_force = gravitational_force(position, mass, earth_position, earth_mass)
    moon_grav_force = gravitational_force(position, mass, moon_position, moon_mass)
    F_x = earth_grav_force[0] + moon_grav_force[0]
    F_y = earth_grav_force[1] + moon_grav_force[1]
    return F_x, F_y

def acceleration_of_satellite(position, mass):
    """Returns the acceleration based on all forces acting upon the body."""
    F_x, F_y = force_on_satellite(position, mass)
    return F_x / mass, F_y / mass

def moonshot(orbital_height, satellite_mass, boost_velocity, initial_y_velocity, 
             time_step=60, max_iterations=5e4, plot_trajectory=False):
    fitness = 0.0
    distance_from_earth_center = orbital_height + earth_radius
    eqb_velocity = math.sqrt(G * earth_mass / distance_from_earth_center)
    
    # Start the simulation.
    # Keep up with the positions of the satellite as it moves.
    position = [(earth_radius + orbital_height, 0.0)] # The initial position of the satellite.
    velocity = [0.0, initial_y_velocity]
    time = 0
    min_distance_from_moon = distance_between(position[-1], moon_position) - moon_radius

    i = 0 
    keep_simulating = True
    rockets_boosted = False

    while keep_simulating:
        # Calculate the acceleration and corresponding change in velocity.
        # (This is effectively the Forward Euler Algorithm.)
        acceleration = acceleration_of_satellite(position[-1], satellite_mass)
        velocity[0] += acceleration[0] * time_step
        velocity[1] += acceleration[1] * time_step 

        # Start the rocket burn:
        # add a boost in the +x direction of 1m/s
        # closest point to the moon
        if position[-1][1] < -100 and position[-1][0] > distance_from_earth_center-100 and not rockets_boosted: 
            launch_point = position[-1]
            velocity[0] += boost_velocity[0]
            velocity[1] += boost_velocity[1]
            rockets_boosted = True

        # Calculate the new position based on the velocity.
        position.append((position[-1][0] + velocity[0] * time_step, 
                         position[-1][1] + velocity[1] * time_step))
        time += time_step

        if i >= max_iterations:
            keep_simulating = False

        distance_from_moon_surface = distance_between(position[-1], moon_position) - moon_radius
        distance_from_earth_surface = distance_between(position[-1], earth_position) - earth_radius
        if distance_from_moon_surface < min_distance_from_moon:
            min_distance_from_moon = distance_from_moon_surface
            
        # See if the satellite crashes into the Moon or the Earth, or
        # if the satellite gets too far away (radio contact is lost).
        if distance_from_moon_surface <= 0:
            fitness += 100000 # penalty of 100,000 km if crash on moon
            keep_simulating = False
        elif distance_from_earth_surface <= 0:
            keep_simulating = False
            fitness -= 100000 # reward of 100,000 km if land on earth
        elif distance_from_earth_surface > 2 * distance_between(earth_position, moon_position): 
            keep_simulating = False #radio contact lost
        i += 1

    # Augment the fitness to include the minimum distance (in km) 
    # that the satellite made it to the Moon (lower without crashing is better).
    fitness += min_distance_from_moon / 1000.0 
    
    # Augment the fitness to include 1% of the total distance
    # traveled by the probe (in km). This means the probe
    # should prefer shorter paths.
    total_distance = 0
    for p, q in pairwise(position):
        total_distance += distance_between(p, q)
    fitness += total_distance / 1000.0 * 0.01

    if plot_trajectory:
        axes = plt.gca()
        earth = Circle(earth_position, earth_radius, facecolor='b', alpha=1)
        moon = Circle(moon_position, moon_radius, facecolor='0.5', alpha=1)
        axes.add_artist(earth)
        axes.add_artist(moon)
        axes.annotate('Earth', xy=earth_position,  xycoords='data',
                      xytext=(0, 1e2), textcoords='offset points',
                      arrowprops=dict(arrowstyle="->"))
        axes.annotate('Moon', xy=moon_position,  xycoords='data',
                      xytext=(0, 1e2), textcoords='offset points',
                      arrowprops=dict(arrowstyle="->"))
        x = [p[0] for p in position] 
        y = [p[1] for p in position]
        cm = pylab.get_cmap('gist_rainbow')
        lines = plt.scatter(x, y, c=range(len(x)), cmap=cm, marker='o', s=2)
        plt.setp(lines, edgecolors='None')  
        plt.axis("equal")
        plt.grid("on")
        projdir = os.path.dirname(os.getcwd())
        print(projdir)
        name = "%s/%s.pdf" % (projdir, str(fitness))
        plt.savefig(name, format="pdf")
        plt.clf()
        
    return fitness


def satellite_generator(random, args):
    chromosome = []
    bounder = args["_ec"].bounder
    # The constraints are as follows:
    #             orbital   satellite   boost velocity      initial y
    #             height    mass        (x,       y)        velocity
    for lo, hi in zip(bounder.lower_bound, bounder.upper_bound):
        chromosome.append(random.uniform(lo, hi))
    return chromosome

def moonshot_evaluator(candidates, args):
    fitness=[]
    for chromosome in candidates:
        orbital_height = chromosome[0]
        satellite_mass = chromosome[1]
        boost_velocity = (chromosome[2], chromosome[3])
        initial_y_velocity = chromosome[4]
        fitness.append(moonshot(orbital_height, satellite_mass, boost_velocity, initial_y_velocity))
    return fitness
    
def custom_observer(population, num_generations, num_evaluations, args):
    best = max(population)
    print("Generations: %d  Evaluations: %d  Best: %s" % (num_generations, num_evaluations, str(best)))

   
#start_main   
rand = Random()
rand.seed(int(time()))
# The constraints are as follows:
#             orbital   satellite   boost velocity      initial y
#             height    mass        (x,       y)        velocity
constraints=((6e6,      10.0,       3e3,    -10000.0,   4000), 
             (8e6,      40.0,       9e3,     10000.0,   6000))

algorithm = ec.EvolutionaryComputation(rand)
algorithm.terminator = terminators.evaluation_termination
algorithm.observer = [observers.file_observer, custom_observer]
algorithm.selector = selectors.tournament_selection
algorithm.replacer = replacers.generational_replacement
algorithm.variator = [variators.blend_crossover, variators.gaussian_mutation]
projdir = os.path.dirname(os.getcwd())

stat_file_name = "%s/data/ec_statistics.csv" % projdir
ind_file_name = "%s/data/ec_individuals.csv" % projdir
stat_file = open(stat_file_name, 'w')
ind_file = open(ind_file_name, 'w')
final_pop = algorithm.evolve(generator=satellite_generator,
                      evaluator=moonshot_evaluator,
                      pop_size=100,
                      maximize=False,
                      bounder=ec.Bounder(constraints[0], constraints[1]),
                      num_selected=100,
                      tourn_size=2,
                      num_elites=1,
                      mutation_rate=0.3,
                      max_evaluations=200,
                      statistics_file=stat_file,
                      individuals_file=ind_file)

stat_file.close()
ind_file.close()

# Sort and print the fittest individual, who will be at index 0.
final_pop.sort(reverse=True)
best = final_pop[0]
components = best.candidate
print("\nFittest individual:")
print(best)
moonshot(components[0], components[1], (components[2], components[3]), components[4], plot_trajectory=True)
#end_main


from random import Random
from time import time
from ecspy import emo
from ecspy import variators
from ecspy import terminators
from ecspy import benchmarks

   
def main(do_plot=True, prng=None):
    if prng is None:
        prng = Random()
        prng.seed(time()) 

    problem = benchmarks.Kursawe(3)
    ea = emo.NSGA2(prng)
    ea.variator = [variators.blend_crossover, variators.gaussian_mutation]
    ea.terminator = terminators.generation_termination
    final_pop = ea.evolve(generator=problem.generator, 
                          evaluator=problem.evaluator, 
                          pop_size=100,
                          maximize=problem.maximize,
                          bounder=problem.bounder,
                          max_generations=80)
    
    if do_plot:
        final_arc = ea.archive
        print('%s Example (%s) Best Solutions: \n' % (ea.__class__.__name__, problem.__class__.__name__))
        for f in final_arc:
            print(f)
        import pylab
        x = []
        y = []
        for f in final_arc:
            x.append(f.fitness[0])
            y.append(f.fitness[1])
        pylab.scatter(x, y, color='b')
        pylab.savefig('%s Example (%s).pdf' % (ea.__class__.__name__, problem.__class__.__name__), format='pdf')
        pylab.show()
    return ea
        
if __name__ == '__main__':
    main()    



"""
    This package allows the creation of evolutionary computations.
    
    .. Copyright (C) 2009  Inspired Intelligence Initiative

    .. This program is free software: you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation, either version 3 of the License, or
       (at your option) any later version.

    .. This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

    .. You should have received a copy of the GNU General Public License
       along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


__all__ = ['analysis', 'archivers', 'benchmarks', 'contrib', 'ec', 'emo', 'evaluators', 'migrators', 
           'observers', 'replacers', 'selectors', 'swarm', 'terminators', 'topologies', 'variators']
__version__ = '1.0'
__author__ = 'Aaron Garrett <aaron.lee.garrett@gmail.com>'
__url__ = 'http://ecspy.googlecode.com'



"""
    This module provides pre-defined migrators for evolutionary computations.

    All migrator functions have the following arguments:
    
    - *random* -- the random number generator object
    - *population* -- the population of Individuals
    - *args* -- a dictionary of keyword arguments
    
    Each migrator function returns the updated population.
    
    .. Copyright (C) 2009  Inspired Intelligence Initiative

    .. This program is free software: you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation, either version 3 of the License, or
       (at your option) any later version.

    .. This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

    .. You should have received a copy of the GNU General Public License
       along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


def default_migration(random, population, args):
    """Do nothing.
    
    This function just returns the existing population with no changes.
    
    """
    return population


"""
    This module provides pre-defined topologies for particle swarms.

    All topology functions have the following arguments:
    
    - *random* -- the random number generator object
    - *population* -- the population of Particles
    - *args* -- a dictionary of keyword arguments
    
    Each topology function returns a list of lists of neighbors
    for each particle in the population. For example, if a swarm
    contained 10 particles, then this function would return a list
    containing 10 lists, each of which contained the neighbors for 
    its corresponding particle in the population.
    
    .. Copyright (C) 2009  Inspired Intelligence Initiative

    .. This program is free software: you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation, either version 3 of the License, or
       (at your option) any later version.

    .. This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

    .. You should have received a copy of the GNU General Public License
       along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


def star_topology(random, population, args):
    """Returns the neighbors using a star topology.
    
    This function sets all particles as neighbors for all other particles.
    This is known as a star topology. The resulting list of lists of 
    neighbors is returned.
    
    .. Arguments:
       random -- the random number generator object
       population -- the population of particles
       args -- a dictionary of keyword arguments
    
    """
    neighbors = [population[:] for _ in range(len(population))]
    return neighbors
    
    
def ring_topology(random, population, args):
    """Returns the neighbors using a ring topology.
    
    This function sets all particles in a specified sized neighborhood
    as neighbors for a given particle. This is known as a ring 
    topology. The resulting list of lists of neighbors is returned.
    
    .. Arguments:
       random -- the random number generator object
       population -- the population of particles
       args -- a dictionary of keyword arguments

    Optional keyword arguments in args:
    
    *neighborhood_size* -- the width of the neighborhood around a 
    particle which determines the size of the neighborhood
    (default 3)
    
    """
    neighborhood_size = args.setdefault('neighborhood_size', 3)

    neighbor_index_start = []
    for index in range(len(population)):
        if index < neighborhood_size // 2:
            neighbor_index_start.append(len(population) - neighborhood_size // 2 + index)
        else:
            neighbor_index_start.append(index - neighborhood_size // 2)
    neighbors = []
    for start in neighbor_index_start:
        n = []
        for i in range(0, neighborhood_size):
            n.append(population[(start + i) % len(population)])
        neighbors.append(n)
    return neighbors
    


'''
Created on Dec 14, 2010

`Self-modifying Cartesian Genetic Programming (SMCGP) is a general purpose,
graph-based, developmental form of Genetic Programming founded on Cartesian Genetic Programming.
In addition to the usual computational functions, it includes functions that can modify
the program encoded in the genotype. This means that programs can be iterated to produce an
infinite sequence of programs (phenotypes) from a single evolved genotype. It also allows programs
to acquire more inputs and produce more outputs during this iteration.`
--Harding,Miller,Banzhaf

Implementation based on the paper
Developments in Cartesian Genetic Programming: self-modifying CGP
by
Simon Harding,Julian F. Miller,Wolfgang Banzhaf



@author: Henrik Rudstrom



'''
import random
import inspect
from ecspy.variators.mutators import mutator


def call_mod(gene, a, b):
    if gene.activated:
        return b
    return a
def get_call_mod(func, call):
    '''called when a modification node is activated'''
    def f(gene, a, b):
        return call(gene,a,b)
    f.__name__ = func.__name__
    f.call_mod = True
    return f

def get_arity(function):
    if hasattr(function, 'input') and function.input:
        return 0
    if hasattr(function, 'call_mod') and function.call_mod:
        return 2
    
    args = len(inspect.getargspec(function).args)
    if hasattr(function, '__self__'):
        return args-1
    return args

class SMCGPGene(object):
    __slots__ = ['function', 'inputs', 'vars', 'activated', '__repr__', '__eq__', '__deepcopy__']
    def __init__(self, function=None, inputs=None, variables=None):
        self.function = function
        self.inputs = inputs
        self.vars = variables
        
    @property
    def activated(self):
        return self.inputs[0] > self.inputs[1]
    
    def __repr__(self):
        varstr = ",".join([str(a)[:5] for a in self.vars])
        return "<%s,%s,%s>" % (self.function, self.inputs, varstr)
    def __eq__(self, other):
        if self.function != other.function:
            return False
        eq = [a == b for a,b in zip(self.inputs, other.inputs)]
        if len(eq) != sum(eq):
            return False
        eq = [a == b for a,b in zip(self.vars, other.vars)]
        if len(eq) != sum(eq):
            return False
        return True
    
    def __deepcopy__(self, memo):
        cpy = self.__class__()
        memo[id(self)] = cpy
        cpy.function = self.function
        cpy.inputs = list(self.inputs)
        cpy.vars = list(self.vars)
        return cpy

class SMCGPEncoding:
    def __init__(self, length, max_arity, levelsback, min_outputs=1, max_modifications=2, variables=3):
        '''
        arguments:
        - *length* -- number of columns in the cgp matrix
        - *max_arity* -- tuple (number of edges pr node, number of data fields pr node) 
        - *levelsback* -- parameter that limits how many columns back a node can connect to, minimum 1
        - *min_outputs* -- minimum outputs needed
        '''
        self.length = length
        self.arity = max_arity
        self.levelsback = levelsback
        self.max_modifications = max_modifications
        self.variables = variables
        self.min_outputs = min_outputs
        self.gene_class = SMCGPGene
        
        
    def set_functions(self, functions, mod_callable=call_mod):
        function_names = []
        user_functions = []
        mod_functions = []
        io_functions = []
        for i, f in enumerate(functions):
            if hasattr(f, 'mod') and f.mod:
                mod_functions.append(f)
            elif hasattr(f, 'input') and f.input:
                io_functions.append(f)
            else:
                user_functions.append(f)
                
        self.functions = io_functions + user_functions + [get_call_mod(f, call_mod) for f in mod_functions]
        self.mod_functions = mod_functions
        #print "IOF", io_functions
        self.io_func_range = (0,len(io_functions))
        self.user_func_range = (self.io_func_range[1],self.io_func_range[1]+len(user_functions))
        self.mod_func_range = (self.user_func_range[1],self.user_func_range[1]+len(mod_functions))
        self.io_func_bias = 0.33
        self.user_func_bias = 0.33
        self.mod_func_bias = 0.33
        self.func_bias_sum = 0.99
        
        self.function_arity = {}
        for i, f in enumerate(self.functions):
            function_names.append(f.__name__)
            self.function_arity[i] = get_arity(f)
            #f.arity = get_arity(f)
        self.function_names = function_names
        
    def get_mod_function(self, index):
        return self.mod_functions[index - self.mod_func_range[0]]     
    
    
    def random_gene(self, col):
        mincol = 1
        if col < 1:
            mincol = 0
        return self.gene_class(
            #random.randint(0, len(self.functions)-1),
            self.random_func(col),
            [random.randint(mincol, min(col, self.levelsback)) for _ in xrange(self.arity)],               
            [random.random() * 10 for _ in xrange(self.variables)]
        )
    
    def generator(self):
        '''
        returns a generator function that generates a candidate
        '''
        
        def gen(random, args):
            matrix = []
            for col in xrange(self.length):
                matrix.append(self.random_gene(col))
            return matrix
        return gen
    
    def random_func(self, v):
        rnd = random.random() * self.func_bias_sum
        def in_range(range):
            return rnd >= range[0] and rnd < range[1]
        bias = self.io_func_bias
        range = self.io_func_range
        if rnd < bias:
            #print range
            return random.randint(range[0], range[1]-1)
        bias += self.user_func_bias
        range = self.user_func_range
        if rnd < bias:
            return random.randint(range[0], range[1]-1)
        bias += self.mod_func_bias
        range = self.mod_func_range
        if rnd < bias:
            return random.randint(range[0], range[1]-1)
        
    def random_edge(self, v):
        return (v + random.randint(1, self.levelsback)) % self.levelsback
    
    def random_var(self, v):
        return random.gauss(v, 1)   

    def mutator(self):        
        def mut(random, cand, args):
            maybe_mutate = lambda v, func: func(v) if random.random() < args['mutation_rate'] else v
            for gene in cand:
                gene.function = maybe_mutate(gene.function, self.random_func)
                gene.inputs = [maybe_mutate(c, self.random_edge) for c in gene.inputs]
                gene.vars = [maybe_mutate(c, self.random_var) for c in gene.vars]
                #gene.vars = [maybe_noise(c, randomize_var) for c in gene.vars] #TODO:get right noise
            return cand
        
        return mutator(mut)       




"""
    .. Copyright (C) 2009  Inspired Intelligence Initiative

    .. This program is free software: you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation, either version 3 of the License, or
       (at your option) any later version.

    .. This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

    .. You should have received a copy of the GNU General Public License
       along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
    
import copy
    
    
def mutator(mutate):
    """Return an ecspy mutator function based on the given function.
    
    This function generator takes a function that operates on only
    one candidate to produce a mutated candidate. The generator 
    handles the iteration over each candidate in the set of offspring.

    The given function ``mutate`` must have the following signature::
    
        mutant = mutate(random, candidate, args)
        
    This function is most commonly used as a function decorator with
    the following usage::
    
        @mutator
        def mutate(random, candidate, args):
            # Implementation of mutation
            
    The generated function also contains an attribute named
    ``single_mutation`` which holds the original mutation function.
    In this way, the original single-candidate function can be
    retrieved if necessary.
    
    """
    def ecspy_mutator(random, candidates, args):
        mutants = list(candidates)
        for i, cs in enumerate(mutants):
            mutants[i] = mutate(random, cs, args)
        return mutants
    ecspy_mutator.__name__ = mutate.__name__
    ecspy_mutator.__dict__ = mutate.__dict__
    ecspy_mutator.__doc__ = mutate.__doc__
    ecspy_mutator.single_mutation = mutate
    return ecspy_mutator
    

@mutator    
def gaussian_mutation(random, candidate, args):
    """Return the mutants created by Gaussian mutation on the candidates.

    This function assumes that the candidate solutions are indexable
    and numeric. It performs Gaussian mutation. This function also 
    makes use of the bounder function as specified in the EC's 
    ``evolve`` method.

    .. Arguments:
       random -- the random number generator object
       candidate -- the candidate solution
       args -- a dictionary of keyword arguments

    Optional keyword arguments in args:
    
    - *mutation_rate* -- the rate at which mutation is performed (default 0.1)
    - *mean* -- the mean used in the Gaussian function (default 0)
    - *stdev* -- the standard deviation used in the Gaussian function
      (default 1.0)
    
    """
    mut_rate = args.setdefault('mutation_rate', 0.1)
    mean = args.setdefault('mean', 0.0)
    stdev = args.setdefault('stdev', 1.0)
    bounder = args['_ec'].bounder
    for i, c in enumerate(candidate):
        if random.random() < mut_rate:
            candidate[i] += random.gauss(mean, stdev)
    candidate = bounder(candidate, args)
    return candidate


@mutator
def bit_flip_mutation(random, candidate, args):
    """Return the mutants produced by bit-flip mutation on the candidates.

    This function assumes that the candidate solution is made of binary 
    values. It performs bit-flip mutation. If a candidate solution contains
    non-binary values, this function leaves it unchanged.

    .. Arguments:
       random -- the random number generator object
       candidate -- the candidate solution
       args -- a dictionary of keyword arguments

    Optional keyword arguments in args:
    
    *mutation_rate* -- the rate at which mutation is performed (default 0.1)
    
    The mutation rate is applied on a bit by bit basis.
    
    """
    rate = args.setdefault('mutation_rate', 0.1)
    if len(candidate) == len([x for x in candidate if x in [0, 1]]):
        for i, c in enumerate(candidate):
            if random.random() < rate:
                candidate[i] = (c + 1) % 2
    return candidate

    
@mutator
def nonuniform_mutation(random, candidate, args):
    bounder = args['_ec'].bounder
    num_gens = args['_ec'].num_generations
    max_gens = args['max_generations']
    strength = args['mutation_strength']
    exponent = (1.0 - num_gens / max_gens) ^ strength
    mutant = copy.copy(candidate)
    for i, (c, lo, hi) in enumerate(zip(candidate, bounder.lower_bound, bounder.upper_bound)):
        if random.random() <= 0.5:
            new_value = c + (hi - c) * (1.0 - random.random() ^ exponent)
        else:
            new_value = c - (c - lo) * (1.0 - random.random() ^ exponent)
        mutant[i] = new_value
    return mutant

    
@mutator
def mptm_mutation(random, candidate, args):
    bounder = args['_ec'].bounder
    strength = args['mutation_strength']
    mutant = copy.copy(candidate)
    for i, (c, lo, hi) in enumerate(zip(candidate, bounder.lower_bound, bounder.upper_bound)):
        t = (c - lo) / (hi - c)
        r = random.random()
        if r < t:
            t_hat = t - t * ((t - r) / t) ^ strength
        elif r == t:
            t_hat = t
        else:
            t_hat = t + (1 - t) * ((r - t) / (1 - t)) ^ strength
        mutant[i] = (1 - t_hat) * lo + t_hat * hi
    return mutant








"""
    This module provides pre-defined variators for evolutionary computations.
    
    All variator functions have the following arguments:
    
    - *random* -- the random number generator object
    - *candidates* -- the candidate solutions
    - *args* -- a dictionary of keyword arguments
    
    Each variator function returns the list of modified individuals.
    
    These variators may make some limited assumptions about the type of
    candidate solutions on which they operate. These assumptions are noted
    in the table below.
    
    +--------------------------------------+----------+-----------+-----------+----------+---------+------+--------+
    | Variator                             | Iterable | Indexable | Sliceable | Listable | Lenable | Real | Binary |
    +======================================+==========+===========+===========+==========+=========+======+========+
    | blend_crossover                      |    X     |           |           |          |    X    |   X  |        |
    +--------------------------------------+----------+-----------+-----------+----------+---------+------+--------+
    | differential_crossover               |    X     |           |           |          |         |   X  |        |
    +--------------------------------------+----------+-----------+-----------+----------+---------+------+--------+
    | n_point_crossover                    |          |           |     X     |     X    |    X    |      |        |
    +--------------------------------------+----------+-----------+-----------+----------+---------+------+--------+
    | simulated_binary_crossover           |    X     |           |           |          |         |   X  |        |
    +--------------------------------------+----------+-----------+-----------+----------+---------+------+--------+
    | uniform_crossover                    |    X     |           |           |          |         |      |        |
    +--------------------------------------+----------+-----------+-----------+----------+---------+------+--------+
    | bit_flip_mutation                    |    X     |     X     |           |          |    X    |      |    X   |
    +--------------------------------------+----------+-----------+-----------+----------+---------+------+--------+
    | gaussian_mutation                    |    X     |     X     |           |          |    X    |   X  |        |
    +--------------------------------------+----------+-----------+-----------+----------+---------+------+--------+
    | default_variation                    |          |           |           |          |         |      |        |
    +--------------------------------------+----------+-----------+-----------+----------+---------+------+--------+
    | estimation_of_distribution_variation |          |     X     |           |          |    X    |   X  |        |
    +--------------------------------------+----------+-----------+-----------+----------+---------+------+--------+
        
    .. Copyright (C) 2009  Inspired Intelligence Initiative

    .. This program is free software: you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation, either version 3 of the License, or
       (at your option) any later version.

    .. This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

    .. You should have received a copy of the GNU General Public License
       along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from crossovers import crossover
from crossovers import blend_crossover
from crossovers import differential_crossover
from crossovers import n_point_crossover
from crossovers import simulated_binary_crossover
from crossovers import uniform_crossover
from mutators import mutator
from mutators import bit_flip_mutation
from mutators import gaussian_mutation
from variators import default_variation
from variators import estimation_of_distribution_variation

__all__ = ['crossover', 'blend_crossover', 'differential_crossover', 'n_point_crossover', 'simulated_binary_crossover', 'uniform_crossover', 
           'mutator', 'bit_flip_mutation', 'gaussian_mutation',
           'default_variation', 'estimation_of_distribution_variation']


'''
Created on Dec 14, 2010
@author: Henrik Rudstrom

`Self-modifying Cartesian Genetic Programming (SMCGP) is a general purpose,
graph-based, developmental form of Genetic Programming founded on Cartesian Genetic Programming.
In addition to the usual computational functions, it includes functions that can modify
the program encoded in the genotype. This means that programs can be iterated to produce an
infinite sequence of programs (phenotypes) from a single evolved genotype. It also allows programs
to acquire more inputs and produce more outputs during this iteration.`
--Harding,Miller,Banzhaf

Implementation based on the paper
Developments in Cartesian Genetic Programming: self-modifying CGP
by
Simon Harding,Julian F. Miller,Wolfgang Banzhaf


'''
import copy



def input(func):
    '''decorator to mark functions as input functions'''
    func.input = True
    return func

@input
def inp(io, gene):
    #print io.inputs, io.pointer, gene
    if io.pointer is None:
        io.pointer = 0
    else:
        io.pointer = (io.pointer + 1) % len(io.inputs)
    return io.inputs[io.pointer] 

@input
def inpp(io, gene):
    if io.pointer is None:
        io.pointer = len(io.inputs) - 1
    else:
        io.pointer = (io.pointer - 1) % io.n_inputs
    return io.inputs[io.pointer] 

def out(self, gene, val):
    '''output function, passes first argument on, in case it is referenced'''
    return val

io_functions = [inp, inpp, out]
class InputPointer:
    '''
    INP, INPP and SKIPINP. When decoding the phenotype graph, a pointer is maintained that refers to an input. 
    If the first occurrence of an active input function is INP it returns the first program input. If the first 
    occurrence is INPP, the last program input is returned. After INP is called it increments the input pointer. 
    INPP decrements the input pointer.
    '''
    def __init__(self, inputs):
        self.inputs = inputs
        self.pointer = None
        self.n_inputs = len(self.inputs)



def bind_modification(enc, gene, index):
    '''bind modification functions with their arguments'''
    mod = enc.get_mod_function(gene.function)
    def f(cand):
        return mod(enc, gene, cand, index)
    f.__name__ = mod.__name__
    return f

class SMCGPPhenotype:
    def __init__(self, encoding, candidate):
        self.encoding = encoding
        self.candidate = candidate
        self._traversal = [(n, inp) for n, inp in self.traverse()]
        self.init_todo_list()
        
    def init_todo_list(self):
        mod_func_index = self.encoding.mod_func_range[0]
        mod_indices = filter(lambda n: n.function >= mod_func_index and n.activated, self.candidate)
        todo = []
        for i, n in enumerate(self.candidate):
            if n.function < mod_func_index or not n.activated:
                continue
            if len(todo) > self.encoding.max_modifications:
                break
            todo.append(bind_modification(self.encoding, n, i))
        self.todo_list = todo 
        return mod_indices    

    def get_inputs(self, node):
        if node == 0:
            return []
        if node >= len(self.candidate):
            print "warning", node
        inps= [(node - i) % node for i in self.candidate[node].inputs]
        for i in inps:
            if i >= len(self.candidate):
                raise Exception() 
        return inps 
        
    def traverse(self, outputs=None):
        '''yields the nodes and their inputs in depth-first ordering'''
        if len(self.candidate) < 1:
            return
        nlist = outputs or self.get_outputs()
        #print "trav", nlist
        seen={} # nodes seen      
        for source in nlist:
            if source in seen: continue
            queue=[source]     # use as LIFO queue
            while queue:
                v=queue[-1]
                if v not in seen:
                    seen[v]=True
                done=1
                inputs = self.get_inputs(v)
                for w in inputs:
                    if w not in seen:
                        queue.append(w)
                        done=0
                        break
                if done == 1:
                    yield v, inputs
                    queue.pop()



    def get_outputs(self):
        '''
        A number of measures need to be taken when the number of OUTPUT nodes 
        does not match the number of program outputs.
        - If there are no OUTPUT nodes in the graph, then the last n nodes in the graph are used
        - If there are more OUTPUT nodes than are required, then the right-most OUTPUT 
            nodes are used until the required number of outputs is reached.
        - If the graph has fewer OUTPUT nodes than are required graph, then nodes are 
        chosen as outputs by moving forwards from the right-most node flagged as an output.
        - If there is a condition where not enough nodes can be used as outputs (as there are 
        not enough nodes in the graph), the individual is labeled as corrupt and is given a 
        bad fitness score to prevent selection.
        '''
        outputs = []
        if len(self.candidate) == 0:
            return []
        for i, c in enumerate(self.candidate):
            if c.function == 2 and i != 0: #first column cant be output
                outputs.append(self.get_inputs(i)[0])
                
        mino = self.encoding.min_outputs
        if len(outputs) == 0:
            outputs = [len(self.candidate)-mino+i for i in xrange(mino)]
        while len(outputs) < self.encoding.min_outputs:
            outputs.append(outputs[-1]+1)
        if len(outputs) > 0:    
            assert max(outputs) < len(self.candidate), "Outputs: %s\nLength of candidate %s" % (outputs, len(self.candidate)) 
        return outputs

        
    def is_valid(self):
        if len(self.candidate) == 0:
            return False
        return True

    def is_active_mod(self, index):
        ''' 
        The input values to nodes are found and the behavior of the node is based on these input values. 
        If the first input is greater or equal to the second, then the graph manipulation function is 
        added to a 'To Do' list of pending modifications and the node returns the first input
        '''
        inputs = self.candidate[1:self.encoding.arity+1]
        return inputs[0] > inputs[1]


    
    def get_modified(self):
        cand = copy.deepcopy(self.candidate)
        #cand = [copy_gene(c) for c in self.candidate]
        for mod in self.todo_list:
            cand = mod(cand)
            if len(cand) == 0:
                break
        return SMCGPPhenotype(self.encoding, cand)
    
    def __call__(self, inputs, values=None):
        functions = list(self.encoding.functions)
        pointer = InputPointer(inputs)
        def input_wrap(func):
            def f(gene):
                return func(pointer, gene)
            f.arity = 0
            f.__name__ = func.__name__
            return f
        #input nodes are always the first in the set
        for i,f in enumerate(functions):
            if not hasattr(f, 'input') or not f.input:
                break
            #print "Func",f
            functions[i] = input_wrap(f)

        if values is None:
            values = {}
        #print self.candidate
        for index, inp in self._traversal:
            gene = self.candidate[index]
            f = gene.function
            if index == 0: #first node can only be input node
                f = f % 2
            func = functions[f]
            arity = self.encoding.function_arity[f]
            args = [gene]+[values[i] for i in inp[0:arity]]
            #print func, arity, args
            val = func(*args)
            values[index] = val
        #print values
        out = [values[inp] for inp in self.get_outputs()] 
        #print out
        return out
    
    def __repr__(self):
        values = {}
        for index, inp in self.traverse():
            node = self.candidate[index]
            name = self.encoding.function_names[node.function]
            arity = self.encoding.function_arity[node.function]
            args = [values[i] for i in inp[0:arity]]
            values[index] = "%s(%s)" % (name, ", ".join(args))
        return "\n".join(["f(%s): %s" % (i, values[i]) for i in self.get_outputs()])



from random import Random
from time import time
from ecspy import ec
from ecspy import observers
from ecspy import terminators
from ecspy import benchmarks


def main(do_plot=True, prng=None): 
    if prng is None:
        prng = Random()
        prng.seed(time()) 
    
    problem = benchmarks.Binary(benchmarks.Schwefel(2), dimension_bits=30)
    
    stat_file = open('ga_statistics.csv', 'w')
    ind_file = open('ga_individuals.csv', 'w')

    ea = ec.GA(prng)
    ea.observer = observers.file_observer
    ea.terminator = terminators.evaluation_termination
    final_pop = ea.evolve(generator=problem.generator,
                          evaluator=problem.evaluator,
                          pop_size=100,
                          maximize=problem.maximize,
                          bounder=problem.bounder,
                          max_evaluations=30000, 
                          statistics_file=stat_file,
                          individuals_file=ind_file,
                          num_elites=1)
    stat_file.close()
    ind_file.close()
    
        
    if do_plot:
        best = max(final_pop)
        print('%s Example (%s) Best Solution: \n%s' % (ea.__class__.__name__, problem.__class__.__name__, str(best)))
        realbest = problem.binary_to_real(best.candidate)
        realfit = best.fitness
        print('Converted as --> %s : %f' % (str(realbest), realfit))
        
        import itertools
        import pylab
        import mpl_toolkits.mplot3d.axes3d as axes3d
        from ecspy import analysis
        
        # Create the points using the original Ackley function.
        num_points = 40
        points = []
        for lb, ub in zip(problem.bounder.lower_bound, problem.bounder.upper_bound):
            points.append([(i / float(num_points)) * (ub - lb) + lb for i in range(num_points)])
        points = itertools.product(*points)
        x = []
        y = []
        for p in points:
            x.append(p[0])
            y.append(p[1])
        z = problem.benchmark.evaluator([[a, b] for a, b in zip(x, y)], {})
        
        fig = pylab.figure()
        ax = axes3d.Axes3D(fig)
        ax.scatter3D(x, y, z)
        ax.scatter3D([realbest[0]], [realbest[1]], [realfit], color='r')
        realopt = problem.benchmark.global_optimum
        ax.scatter3D([realopt[0]], [realopt[1]], problem.benchmark.evaluator([realopt], {}), color='g')
        ax.set_xlabel('X')
        ax.set_ylabel('Y')
        ax.set_zlabel('Fitness')
        pylab.savefig('%s Example (%s).pdf' % (ea.__class__.__name__, problem.__class__.__name__), format='pdf')
        analysis.generation_plot('ga_statistics.csv', errorbars=False)
    return ea
            
if __name__ == '__main__':
    main()


"""
    This module provides pre-defined replacers for evolutionary computations.
    
    All replacer functions have the following arguments:
    
    - *random* -- the random number generator object
    - *population* -- the population of individuals
    - *parents* -- the list of parent individuals
    - *offspring* -- the list of offspring individuals
    - *args* -- a dictionary of keyword arguments
    
    Each replacer function returns the list of surviving individuals.

    .. Copyright (C) 2009  Inspired Intelligence Initiative

    .. This program is free software: you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation, either version 3 of the License, or
       (at your option) any later version.

    .. This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

    .. You should have received a copy of the GNU General Public License
       along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import math


def default_replacement(random, population, parents, offspring, args):
    """Replaces entire population with offspring.
    
    .. Arguments:
       random -- the random number generator object
       population -- the population of individuals
       parents -- the list of parent individuals
       offspring -- the list of offspring individuals
       args -- a dictionary of keyword arguments
    
    """
    return offspring

    
def truncation_replacement(random, population, parents, offspring, args):
    """Replaces population with the best of the population and offspring.
    
    This function performs truncation replacement, which means that
    the entire existing population is replaced by the best from among
    the current population and offspring, keeping the existing population
    size fixed. This is similar to so called 'plus' replacement in the 
    evolution strategies literature, except that 'plus' replacement 
    considers only parents and offspring for survival. However, if the
    entire population are parents (which is often the case in ES), then
    truncation replacement and plus-replacement are equivalent approaches.
    
    .. Arguments:
       random -- the random number generator object
       population -- the population of individuals
       parents -- the list of parent individuals
       offspring -- the list of offspring individuals
       args -- a dictionary of keyword arguments
    
    """
    pool = list(population)
    pool.extend(list(offspring))
    pool.sort(reverse=True)
    return pool[:len(population)]

    
def steady_state_replacement(random, population, parents, offspring, args):
    """Performs steady-state replacement for the offspring.
    
    This function performs steady-state replacement, which means that
    the offspring replace the least fit individuals in the existing
    population, even if those offspring are less fit than the individuals
    that they replace.
    
    .. Arguments:
       random -- the random number generator object
       population -- the population of individuals
       parents -- the list of parent individuals
       offspring -- the list of offspring individuals
       args -- a dictionary of keyword arguments
    
    """
    off = list(offspring)
    pop = list(population)
    pop.sort()
    num_to_replace = min(len(off), len(pop))
    pop[:num_to_replace] = off[:num_to_replace]
    return pop


def generational_replacement(random, population, parents, offspring, args):
    """Performs generational replacement with optional weak elitism.
    
    This function performs generational replacement, which means that
    the entire existing population is replaced by the offspring,
    truncating to the population size if the number of offspring is 
    larger. Weak elitism may also be specified through the num_elites
    keyword argument in args. If this is used, the best num_elites
    individuals in the current population are allowed to survive if
    they are better than the worst num_elites offspring.
    
    .. Arguments:
       random -- the random number generator object
       population -- the population of individuals
       parents -- the list of parent individuals
       offspring -- the list of offspring individuals
       args -- a dictionary of keyword arguments

    Optional keyword arguments in args:
    
    *num_elites* -- number of elites to consider (default 0)
    
    """
    num_elites = args.setdefault('num_elites', 0)
    off = list(offspring)
    pop = list(population)
    pop.sort(reverse=True)
    off.extend(pop[:num_elites])
    off.sort(reverse=True)
    survivors = off[:len(population)]
    return survivors


def random_replacement(random, population, parents, offspring, args):
    """Performs random replacement with optional weak elitism.
    
    This function performs random replacement, which means that
    the offspring replace random members of the population, keeping
    the population size constant. Weak elitism may also be specified 
    through the num_elites keyword argument in args. If this is used, 
    the best num_elites individuals in the current population are 
    allowed to survive if they are better than the worst num_elites 
    offspring.
    
    .. Arguments:
       random -- the random number generator object
       population -- the population of individuals
       parents -- the list of parent individuals
       offspring -- the list of offspring individuals
       args -- a dictionary of keyword arguments

    Optional keyword arguments in args:
    
    *num_elites* -- number of elites to consider (default 0)
    
    """
    num_elites = args.setdefault('num_elites', 0)
    off = list(offspring)
    pop = list(population)
    pop.sort(reverse=True)
    num_to_replace = min(len(off), len(pop) - num_elites) 
    valid_indices = range(num_elites, len(pop))
    rep_index = random.sample(valid_indices, num_to_replace)
    for i, repind in enumerate(rep_index):
        pop[repind] = off[i]
    return pop


def plus_replacement(random, population, parents, offspring, args):
    """Performs 'plus' replacement with optional adaptive mutation.
    
    This function performs 'plus' replacement, which means that
    the entire existing population is replaced by the best
    population-many elements from the combined set of parents and 
    offspring. Adaptive mutation can also be specified here through 
    the use_one_fifth_rule keyword argument in args. This adaptive
    mutation attempts to keep the rate of viable offspring near 20%.
    If the number of successful offspring is below 20%, the mutation
    rate is reduced by 20% (to allow more exploitation). If the 
    number of successful offspring is above 20%, the mutation rate
    is increased by 20% (to allow more exploration).
    
    .. Arguments:
       random -- the random number generator object
       population -- the population of individuals
       parents -- the list of parent individuals
       offspring -- the list of offspring individuals
       args -- a dictionary of keyword arguments

    Optional keyword arguments in args:
    
    *use_one_fifth_rule* -- whether the 1/5 rule should be used (default False)
    
    """
    use_one_fifth_rule = args.setdefault('use_one_fifth_rule', False)
    pool = list(offspring)
    pool.extend(list(parents))
    pool.sort(reverse=True)
    survivors = pool[:len(population)]
    if use_one_fifth_rule:
        count = len([x for x in offspring if x in survivors])
        rate = count / float(len(offspring))
        if rate < 0.2:
            try:
                args['mutation_rate'] = args['mutation_rate'] * 0.8
            except KeyError:
                args['use_one_fifth_rule'] = False
        elif rate > 0.2:
            try:
                args['mutation_rate'] = args['mutation_rate'] * 1.2
            except KeyError:
                args['use_one_fifth_rule'] = False            
    return survivors


def comma_replacement(random, population, parents, offspring, args):
    """Performs 'comma' replacement.
    
    This function performs 'comma' replacement, which means that
    the entire existing population is replaced by the best
    population-many elements from the offspring. This function
    makes the assumption that the size of the offspring is at 
    least as large as the original population. Otherwise, the
    population size will not be constant.
    
    .. Arguments:
       random -- the random number generator object
       population -- the population of individuals
       parents -- the list of parent individuals
       offspring -- the list of offspring individuals
       args -- a dictionary of keyword arguments

    Optional keyword arguments in args:
    
    *use_one_fifth_rule* -- whether the 1/5 rule should be used (default False)
       
    """
    pool = list(offspring)
    pool.sort(reverse=True)
    survivors = pool[:len(population)]
    return survivors


def crowding_replacement(random, population, parents, offspring, args):
    """Performs crowding replacement as a form of niching.
    
    This function performs crowding replacement, which means that
    the members of the population are replaced one-at-a-time with
    each of the offspring. A random sample of `crowding_distance`
    individuals is pulled from the current population, and the
    closest individual to the current offspring (where "closest"
    is determined by the `distance_function`) is replaced by that
    offspring. It is possible for one offspring to replace an 
    earlier offspring in the same generation, given the random
    sample that is taken of the current survivors for each offspring.
    
    .. Arguments:
       random -- the random number generator object
       population -- the population of individuals
       parents -- the list of parent individuals
       offspring -- the list of offspring individuals
       args -- a dictionary of keyword arguments

    Optional keyword arguments in args:    
    
    - *distance_function* -- a function that accepts two candidate 
      solutions and returns the distance between them (default 
      Euclidean L2 distance)
    - *crowding_distance* -- a positive integer representing the 
      number of closest solutions to consider as a "crowd" (default 2)
       
    """
    def distance(x, y):
        return math.sqrt(sum([(a - b)**2 for a, b in zip(x, y)]))
    try:
        distance_function = args['distance_function']
    except KeyError:
        distance_function = distance
        args['distance_function'] = distance_function
    crowding_distance = args.setdefault('crowding_distance', 2)
    survivors = population[:]
    for o in offspring:
        pool = random.sample(survivors, crowding_distance)
        closest = min(pool, key=lambda x: distance_function(o.candidate, x.candidate))
        if o.fitness > closest.fitness:
            survivors.remove(closest)
            survivors.append(o)
    return survivors



    
#-------------------------------------------
# Algorithm-specific Replacement Strategies
#-------------------------------------------
    
def simulated_annealing_replacement(random, population, parents, offspring, args):
    """Replaces population using the simulated annealing schedule.
    
    This function performs simulated annealing replacement based
    on a temperature and a cooling rate. These can be specified
    by the keyword arguments 'temperature', which should be the
    initial temperature, and 'cooling_rate', which should be the
    coefficient by which the temperature is reduced. If these
    keyword arguments are not present, then the function will
    attempt to base the cooling schedule either on the ratio of 
    evaluations to the maximum allowed evaluations or on the 
    ratio of generations to the maximum allowed generations. 
    Each of these rations is of the form (max - current)/max
    so that the cooling schedule moves smoothly from 1 to 0.
    
    .. Arguments:
       random -- the random number generator object
       population -- the population of individuals
       parents -- the list of parent individuals
       offspring -- the list of offspring individuals
       args -- a dictionary of keyword arguments
    
    Optional keyword arguments in args:    
    
    - *temperature* -- the initial temperature
    - *cooling_rate* -- a real-valued coefficient in the range (0, 1) 
      by which the temperature should be reduced 
    
    """
    try:
        temp = args['temperature']
        cooling_rate = args['cooling_rate']
        temp = temp * cooling_rate
        args['temperature'] = temp
    except KeyError:
        try:
            num_evals = args['_ec'].num_evaluations
            max_evals = args['max_evaluations']
            temp = float(max_evals - num_evals) / float(max_evals)
        except KeyError:
            num_gens = args['_ec'].num_generations
            max_gens = args['max_generations']
            temp = 1 - float(max_gens - num_gens) / float(max_gens)
        
    new_pop = []
    for p, o in zip(parents, offspring):
        if o >= p:
            new_pop.append(o)
        elif random.random() < math.exp(-abs(p.fitness - o.fitness) / float(temp)):
            new_pop.append(o)
        else:
            new_pop.append(p)
            
    return new_pop

    
def nsga_replacement(random, population, parents, offspring, args):
    """Replaces population using the non-dominated sorting technique from NSGA-II.
    
    .. Arguments:
       random -- the random number generator object
       population -- the population of individuals
       parents -- the list of parent individuals
       offspring -- the list of offspring individuals
       args -- a dictionary of keyword arguments
    
    """
    survivors = []
    combined = population[:]
    combined.extend(offspring[:])
    
    # Perform the non-dominated sorting to determine the fronts.
    fronts = []
    pop = set(range(len(combined)))
    while len(pop) > 0:
        front = []
        for p in pop:
            dominated = False
            for q in pop:
                if combined[p] < combined[q]:
                    dominated = True
                    break
            if not dominated:
                front.append(p)
        fronts.append([dict(individual=combined[f], index=f) for f in front])
        pop = pop - set(front)
    
    # Go through each front and add all the elements until doing so
    # would put you above the population limit. At that point, fall
    # back to the crowding distance to determine who to put into the
    # next population. Individuals with higher crowding distances
    # (i.e., more distance between neighbors) are preferred.
    for i, front in enumerate(fronts):
        if len(survivors) + len(front) > len(population):
            # Determine the crowding distance.
            distance = [0 for _ in range(len(combined))]
            individuals = front[:]
            num_individuals = len(individuals)
            num_objectives = len(individuals[0]['individual'].fitness)
            for obj in range(num_objectives):
                individuals.sort(key=lambda x: x['individual'].fitness[obj])
                distance[individuals[0]['index']] = float('inf')
                distance[individuals[-1]['index']] = float('inf')
                for i in range(1, num_individuals-1):
                    distance[individuals[i]['index']] = (distance[individuals[i]['index']] + 
                                                         (individuals[i+1]['individual'].fitness[obj] - 
                                                          individuals[i-1]['individual'].fitness[obj]))
                
            crowd = [dict(dist=distance[f['index']], index=f['index']) for f in front]
            crowd.sort(key=lambda x: x['dist'], reverse=True)
            last_rank = [combined[c['index']] for c in crowd]
            r = 0
            num_added = 0
            num_left_to_add = len(population) - len(survivors)
            while r < len(last_rank) and num_added < num_left_to_add:
                if last_rank[r] not in survivors:
                    survivors.append(last_rank[r])
                    num_added += 1
                r += 1
            # If we've filled out our survivor list, then stop.
            # Otherwise, process the next front in the list.
            if len(survivors) == len(population):
                break
        else:
            for f in front:
                if f['individual'] not in survivors:
                    survivors.append(f['individual'])
    return survivors

    
def paes_replacement(random, population, parents, offspring, args):
    """Replaces population using the Pareto Archived Evolution Strategy method.
    
    .. Arguments:
       random -- the random number generator object
       population -- the population of individuals
       parents -- the list of parent individuals
       offspring -- the list of offspring individuals
       args -- a dictionary of keyword arguments
    
    """
    archive = args['_ec'].archive
    archiver = args['_ec'].archiver
        
    survivors = []
    for p, o in zip(parents, offspring):
        if o == p:
            survivors.append(p)
        elif o in archive:
            survivors.append(p)
        elif o > p:
            archive = archiver(random, [o], archive, args)
            survivors.append(o)
        elif o >= p:
            for a in archive:
                if o > a or o < a:
                    break
            if o >= a:
                archive = archiver(random, [o], archive, args)
                if o > a or archiver.grid_population[o.grid_location] <= archiver.grid_population[p.grid_location]:
                    survivors.append(o)
                else:
                    survivors.append(p)
            else:
                survivors.append(p)
        else:
            survivors.append(p)
    return survivors



'''
Created on Dec 11, 2010
@author: Henrik Rudstrom

example self modification function set as defined in
Table 1 of "Developments in Cartesian Genetic Programming: self-modifying CGP"


'''
import copy


def self_modification(name=None):
    '''decorator for self modification functions'''
    
    def mod(func):
        if name != None:
            func.__name__ = name 
        func.mod = True
        return func
    return mod

def val_to_index(cand, val, end=False):
    return max(0,min(len(cand)-int(not end), int(val)))

def decode_relative_range(gene, cand, x):
    '''decodes gene as range (P0 + x) and (P0 + x + P1)'''
    #gene = cand[index]
    start = val_to_index(cand,int(gene.vars[0]) + x)
    end = val_to_index(cand,start + int(gene.vars[1]), True)
    return (min(start, end), max(start, end)) 

def decode_relative_index(gene, cand, x, input_index):
    #gene = cand[index]
    return val_to_index(cand,int(gene.vars[input_index]) + x) 



def insert_range(cand, new, i):
    return cand[:i]+new+cand[i:]

# Self modification functions. 
# see 
@self_modification('ADD')
def add(encoding, gene, cand, index):
    s,e = decode_relative_range(gene,cand,index)
    new = [encoding.random_gene(col) for col in xrange(int(gene.vars[1]))]
    #print "new",len(new)
    return insert_range(cand, new, s)

@self_modification('MOV')
def move(encoding, gene, cand, index):
    s,e = decode_relative_range(gene,cand,index)
    ins = decode_relative_index(gene,cand,index,2)
    if ins >= s:
        cand = cand[:s]+cand[e:ins]+cand[s:e]+cand[ins:]
    else:
        cand = cand[:ins]+cand[s:e]+cand[ins:s]+cand[e:]
    return cand

@self_modification('DEL')
def delete(encoding, gene, cand, index):
    s,e = decode_relative_range(gene,cand,index)
    return cand[:s]+cand[e:]

@self_modification('DUP')
def duplicate(encoding, gene, cand, index):
    s,e = decode_relative_range(gene,cand,index)
    ins = decode_relative_index(gene,cand,index,2)
    return cand[:ins]+cand[s:e]+cand[ins:]

@self_modification('OVR')
def overwrite(encoding, gene, cand, index):
    s,e = decode_relative_range(gene,cand,index)
    for i in xrange(s, e):
        ofsi = decode_relative_index(gene,cand,0,2)
        cand[ofsi] = cand[i]
    return cand

@self_modification('DU3')
def duplicate_and_shift(encoding, gene, cand, index):
    s,e = decode_relative_range(gene,cand,index)
    ins = decode_relative_index(gene,cand,index,2)
    connection_offset = -(s - ins)
    #print "OFS", connection_offset
    duped = [copy.deepcopy(c) for c in cand[s:e]]
    for d in duped:
        for i in xrange(len(d.inputs)):
            d.inputs[i] += connection_offset
    return cand[:ins]+duped+cand[ins:]
    
@self_modification('DU4')
def duplicate_and_scale(encoding, gene, cand, index):
    s,e = decode_relative_range(gene,cand,index)
    ins = e#decode_relative_index(gene,cand,index,2)
    duped = [copy.deepcopy(c) for c in cand[s:e]]
    for d in duped:
        for i in xrange(len(d.inputs)):
            d.inputs[i] = int(d.inputs[i] * gene.vars[2])
    return cand[:ins]+duped+cand[ins:]

@self_modification('SHIFTCONNECTION')
def shift_connections(encoding, gene, cand, x):
    '''Starting at node index (P0 + x), add P2 to the values of the cij of next P1 nodes'''
    s, e = decode_relative_range(gene,cand,x)
    for i in xrange(s,e):
        for j in xrange(len(cand[i].inputs)):
            cand[i].inputs[j] += int(gene.vars[2])
    return cand

@self_modification('MULTCONNECTION')
def scale_connections(encoding, gene, cand, x):
    '''Starting at node index (P0 + x), multiply the cij of the next P1 nodes by P2'''
    s, e = decode_relative_range(gene,cand,x)
    for i in xrange(s,e):
        for j in xrange(len(cand[i].inputs)):
            cand[i].inputs[j] = int(cand[i].inputs[j] * gene.vars[2])
    return cand

@self_modification('CHC')      
def change_connection(encoding, gene, cand, x):
    '''Change the (P1 mod 3)th connection of node P0 to P2'''
    inp = int(gene.vars[1]) % len(gene.inputs)
    i = val_to_index(cand, gene.vars[0])
    cand[i].inputs[inp] = 1+abs(int(gene.vars[2]) % (i+1))
    return cand

@self_modification('CHF')
def change_function(encoding, gene, cand, x):
    '''Change the function of node P0 to the function associated with P1'''
    i1 = val_to_index(cand, gene.vars[0])
    i2 = val_to_index(cand, gene.vars[1])
    cand[i1].function = cand[i2].function
    return cand

@self_modification('CHP')
def change_parameter(encoding, gene, cand, x):
    '''Change the (P1 mod 3)th parameter of node P0 to P2'''
    
    var = int(gene.vars[1]) % len(gene.vars)
    i = val_to_index(cand, gene.vars[0])
    #print i, var, len(gene.vars), len(cand) 
    cand[i].vars[var] = int(gene.vars[2])
    return cand


all_functions = [add, move, delete, duplicate, overwrite, 
                 duplicate_and_shift, duplicate_and_scale, 
                 shift_connections, scale_connections, 
                 change_connection, change_function, change_parameter]


#start_imports
from random import Random
from time import time
from time import sleep
from ecspy import ec
from ecspy import observers
from ecspy import replacers
from ecspy import selectors
from ecspy import terminators
from ecspy import variators
from Tkinter import *
import itertools
#end_imports


def area(p):
    return 0.5 * abs(sum([x0*y1 - x1*y0 for ((x0, y0), (x1, y1)) in segments(p)]))

def segments(p):
    return zip(p, p[1:] + [p[0]])

def generate_polygon(random, args):
    size = args.get('num_vertices', 6)
    return [(random.uniform(-1, 1), random.uniform(-1, 1)) for i in xrange(size)]

def evaluate_polygon(candidates, args):
    fitness = []
    for cs in candidates:
        fit = area(cs)
        fitness.append(fit)
    return fitness

#start_bounder    
def bound_polygon(candidate, args):
    for i, c in enumerate(candidate):
        x = max(min(c[0], 1), -1)
        y = max(min(c[1], 1), -1)
        candidate[i] = (x, y)
    return candidate
bound_polygon.lower_bound = itertools.repeat(-1)
bound_polygon.upper_bound = itertools.repeat(1)
#end_bounder
    
def mutate_polygon(random, candidates, args):
    mut_rate = args.setdefault('mutation_rate', 0.1)
    bounder = args['_ec'].bounder
    cs_copy = list(candidates)
    for i, cs in enumerate(cs_copy):
        for j, (c, lo, hi) in enumerate(zip(cs, bounder.lower_bound, bounder.upper_bound)):
            if random.random() < mut_rate:
                x = c[0] + random.gauss(0, 1) * (hi - lo)
                y = c[1] + random.gauss(0, 1) * (hi - lo)
                cs_copy[i][j] = (x, y)
        cs_copy[i] = bounder(cs_copy[i], args)
    return cs_copy
        
def polygon_observer(population, num_generations, num_evaluations, args):
    try:
        canvas = args['canvas']
    except KeyError:
        canvas = Canvas(Tk(), bg='white', height=400, width=400)
        args['canvas'] = canvas
        
    # Get the best polygon in the population.
    poly = population[0].candidate
    coords = [(100*x + 200, -100*y + 200) for (x, y) in poly]
    old_polys = canvas.find_withtag('poly')
    for p in old_polys:
        canvas.delete(p)
    old_rects = canvas.find_withtag('rect')
    for r in old_rects:
        canvas.delete(r)
    old_verts = canvas.find_withtag('vert')
    for v in old_verts:
        canvas.delete(v)
        
    canvas.create_rectangle(100, 100, 300, 300, fill='', outline='yellow', width=6, tags='rect')
    canvas.create_polygon(coords, fill='', outline='black', width=2, tags='poly')
    vert_radius = 3
    for (x, y) in coords:
        canvas.create_oval(x-vert_radius, y-vert_radius, x+vert_radius, y+vert_radius, fill='blue', tags='vert')
    canvas.pack()
    canvas.update()
    print('%d evaluations' % num_evaluations)
    sleep(0.05)

#start_main
rand = Random()
rand.seed(int(time()))
my_ec = ec.EvolutionaryComputation(rand)
my_ec.selector = ec.selectors.tournament_selection
my_ec.variator = [ec.variators.uniform_crossover, mutate_polygon]
my_ec.replacer = ec.replacers.steady_state_replacement
my_ec.observer = polygon_observer
my_ec.terminator = [terminators.evaluation_termination, terminators.average_fitness_termination]
window = Tk()
window.title('Evolving Polygons')
can = Canvas(window, bg='white', height=400, width=400)
can.pack()

final_pop = my_ec.evolve(generator=generate_polygon,
                         evaluator=evaluate_polygon,
                         pop_size=100,
                         bounder=bound_polygon,
                         max_evaluations=5000,
                         num_selected=2,
                         mutation_rate=0.25,
                         num_vertices=3,
                         canvas=can)
# Sort and print the best individual, who will be at index 0.
final_pop.sort(reverse=True)
print('Terminated due to %s.' % my_ec.termination_cause)
print(final_pop[0])
sleep(5)
#end_main



"""
    This module provides capabilities for creating swarm intelligence algorithms.
    
    .. Copyright (C) 2009  Inspired Intelligence Initiative

    .. This program is free software: you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation, either version 3 of the License, or
       (at your option) any later version.

    .. This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

    .. You should have received a copy of the GNU General Public License
       along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


import math
import copy
from ecspy import ec
from ecspy import archivers
from ecspy import observers
from ecspy import terminators
from ecspy import topologies



class PSO(ec.EvolutionaryComputation):
    """Represents a basic particle swarm optimization algorithm.
    
    This class is built upon the ``EvolutionaryComputation`` class making
    use of an external archive and maintaining the population at the previous
    timestep, rather than a velocity. This approach was outlined in 
    (Deb and Padhye, "Development of Efficient Particle Swarm Optimizers by
    Using Concepts from Evolutionary Algorithms", GECCO 2010, pp. 55--62).
    
    Public Attributes:
    
    - *topology* -- the neighborhood topology (default topologies.star_topology)
    
    Optional keyword arguments in ``evolve`` args parameter:
    
    - *inertia* -- the inertia constant to be used in the particle 
      updating
    - *cognitive_rate* -- the rate at which the particle's current 
      position influences its movement (default 2.1)
    - *social_rate* -- the rate at which the particle's neighbors 
      influence its movement (default 2.1)
    
    """
    def __init__(self, random):
        ec.EvolutionaryComputation.__init__(self, random)
        self.topology = topologies.star_topology
        self._previous_population = []
        self.selector = self._swarm_selector
        self.replacer = self._swarm_replacer
        self.variator = self._swarm_variator
        
    def _swarm_archiver(self, random, population, archive, args):
        if len(archive) == 0:
            return population[:]
        else:
            new_archive = []
            for i, (p, a) in enumerate(zip(population[:], archive[:])):
                if p < a:
                    new_archive.append(a)
                else:
                    new_archive.append(p)
            return new_archive
        
    def _swarm_variator(self, random, candidates, args):
        inertia = args.setdefault('inertia', 0.5)
        cognitive_rate = args.setdefault('cognitive_rate', 1.0)
        social_rate = args.setdefault('social_rate', 1.0)
        if len(self.archive) == 0:
            self.archive = self.population[:]
        if len(self._previous_population) == 0:
            self._previous_population = self.population[:]
        neighbors = self.topology(self._random, self.archive, args)
        offspring = []
        for x, xprev, pbest, hood in zip(self.population, self._previous_population, self.archive, neighbors):
            nbest = max(hood)
            particle = []
            for xi, xpi, pbi, nbi in zip(x.candidate, xprev.candidate, pbest.candidate, nbest.candidate):
                value = xi + inertia * (xi - xpi) + cognitive_rate * random.random() * (pbi - xi) + social_rate * random.random() * (nbi - xi)
                particle.append(value)
            particle = self.bounder(particle, args)
            offspring.append(particle)
        return offspring
        
    def _swarm_selector(self, random, population, args):
        return population
        
    def _swarm_replacer(self, random, population, parents, offspring, args):
        self._previous_population = population[:]
        return offspring

    



from random import Random
from time import time
from ecspy import ec
from ecspy import terminators
from ecspy import observers
from ecspy import benchmarks


def main(do_plot=True, prng=None):
    if prng is None:
        prng = Random()
        prng.seed(time()) 
    
    problem = benchmarks.Griewank(2)
    ea = ec.DEA(prng)
    ea.terminator = terminators.evaluation_termination
    final_pop = ea.evolve(generator=problem.generator, 
                          evaluator=problem.evaluator, 
                          pop_size=100, 
                          bounder=problem.bounder,
                          maximize=problem.maximize,
                          max_evaluations=30000)
                          
    if do_plot:
        best = max(final_pop) 
        print('%s Example (%s) Best Solution: \n%s' % (ea.__class__.__name__, problem.__class__.__name__, str(best)))
        import itertools
        import pylab
        import mpl_toolkits.mplot3d.axes3d as axes3d
        num_points = 60
        points = []
        for lb, ub in zip(problem.bounder.lower_bound, problem.bounder.upper_bound):
            points.append([(i / float(num_points)) * (ub - lb) + lb for i in range(num_points)])
        points = itertools.product(*points)
        x = []
        y = []
        for p in points:
            x.append(p[0])
            y.append(p[1])
        z = problem.evaluator([[a, b] for a, b in zip(x, y)], {})
        fig = pylab.figure()
        ax = axes3d.Axes3D(fig)
        ax.scatter3D(x, y, z)
        ax.scatter3D([best.candidate[0]], [best.candidate[1]], [best.fitness], color='r')
        ax.scatter3D([problem.global_optimum[0]], [problem.global_optimum[1]], problem.evaluator([problem.global_optimum], {}), color='g')
        ax.set_xlabel('X')
        ax.set_ylabel('Y')
        ax.set_zlabel('Fitness')
        pylab.savefig('%s Example (%s).pdf' % (ea.__class__.__name__, problem.__class__.__name__), format='pdf')
        pylab.show()
    return ea

if __name__ == '__main__':
    main()        



"""
    .. Copyright (C) 2009  Inspired Intelligence Initiative

    .. This program is free software: you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation, either version 3 of the License, or
       (at your option) any later version.

    .. This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

    .. You should have received a copy of the GNU General Public License
       along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import copy


def default_variation(random, candidates, args):
    """Return the set of candidates without variation.

    .. Arguments:
       random -- the random number generator object
       candidates -- the candidate solutions
       args -- a dictionary of keyword arguments
    
    """
    return candidates

    
def estimation_of_distribution_variation(random, candidates, args):
    """Return the offspring produced using estimation of distribution.

    This function assumes that the candidate solutions are iterable 
    objects containing real values. It creates a statistical model 
    based on the set of candidates. The offspring are then generated 
    from this model. This function also makes use of the bounder
    function as specified in the EC's ``evolve`` method.

    .. Arguments:
       random -- the random number generator object
       candidates -- the candidate solutions
       args -- a dictionary of keyword arguments
    
    Optional keyword arguments in args:
    
    - *num_offspring* -- the number of offspring to create (default 1)
    
    """
    num_offspring = args.setdefault('num_offspring', 1)
    bounder = args['_ec'].bounder
    
    cs_copy = list(candidates)
    num_genes = max([len(x) for x in cs_copy])
    genes = [[x[i] for x in cs_copy] for i in range(num_genes)] 
    mean = [float(sum(x)) / float(len(x)) for x in genes]
    stdev = [sum([(x - m)**2 for x in g]) / float(len(g) - 1) for g, m in zip(genes, mean)]
    offspring = []
    for _ in range(num_offspring):
        child = copy.copy(cs_copy[0])
        for i, (m, s) in enumerate(zip(mean, stdev)):
            child[i] = m + random.gauss(0, s)
        child = bounder(child, args)
        offspring.append(child)
        
    return offspring



import psyco; psyco.full()
import unittest, sys, os, random
pth = os.path.split( os.path.split( os.path.abspath(__file__) )[0] )[0] 
sys.path.append( pth )

from examples import custom_ec_example
from examples import dea_example
from examples import eda_example
from examples import es_example
from examples import ga_example
from examples import niche_example
from examples import nsga_example
from examples import paes_example
from examples import pso_example
from examples import sa_example

prng = random.Random()
prng.seed(12345) 

class Custom_EC_Test(unittest.TestCase):
    def test(self):
        ea = custom_ec_example.main(do_plot=False, prng=prng)
        best = max(ea.population)
        assert best.fitness < 0.19

class DEA_Test(unittest.TestCase):
    def test(self):
        dea = dea_example.main(do_plot=False, prng=prng)
        best = max(dea.population)
        assert best.fitness < 0.3

class EDA_Test(unittest.TestCase):
    def test(self):
        eda = eda_example.main(do_plot=False, prng=prng)
        best = max(eda.population)
        assert 79 < best.fitness < 82

class ES_Test(unittest.TestCase):
    def test(self):
        es = es_example.main(do_plot=False, prng=prng)
        best = max(es.population)
        assert best.fitness < 0.02

class GA_Test(unittest.TestCase):
    def test(self):
        ga = ga_example.main(do_plot=False, prng=prng)
        best = max(ga.population)
        assert best.fitness < 0.004
        
class NSGA_Test(unittest.TestCase):
    def test(self):
        nsga = nsga_example.main(do_plot=False, prng=prng)
        fitnesses = [a.fitness for a in nsga.archive]
        assert all([(-21 < f[0] < -12) and (-12 < f[1] < 1) for f in fitnesses])

class PAES_Test(unittest.TestCase):
    def test(self):
        paes = paes_example.main(do_plot=False, prng=prng)
        fitnesses = [a.fitness for a in paes.archive]
        assert all([(-21 < f[0] < -12) and (-12 < f[1] < 1) for f in fitnesses])

class PSO_Test(unittest.TestCase):
    def test(self):
        pso = pso_example.main(do_plot=False, prng=prng)
        best = max(pso.population)
        assert best.fitness < 2

class Niche_Test(unittest.TestCase):
    def test(self):
        ea = niche_example.main(do_plot=False, prng=prng)
        candidates = [p.candidate[0] for p in ea.population]
        self.assertTrue(any(map(lambda x: 1 < x < 2, candidates)), 'expected a solution in [1, 2]')
        self.assertTrue(any(map(lambda x: 7 < x < 8, candidates)), 'expected a solution in [7, 8]')
        self.assertTrue(any(map(lambda x: 14 < x < 15, candidates)), 'expected a solution in [14, 15]')
        self.assertTrue(any(map(lambda x: 20 < x < 21, candidates)), 'expected a solution in [20, 21]')
        

if __name__ == '__main__':
    unittest.main()



"""
    .. Copyright (C) 2009  Inspired Intelligence Initiative

    .. This program is free software: you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation, either version 3 of the License, or
       (at your option) any later version.

    .. This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

    .. You should have received a copy of the GNU General Public License
       along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import math
import copy


def crossover(cross):
    """Return an ecspy crossover function based on the given function.

    This function generator takes a function that operates on only
    two parent candidates to produce an iterable list of offspring
    (typically two). The generator handles the pairing of selected
    parents and collecting of all offspring.

    The generated function chooses every odd candidate as a 'mom' and
    every even as a 'dad' (discounting the last candidate if there is
    and odd number). For each mom-dad pair, offspring are produced via
    the `cross` function.

    The given function ``cross`` must have the following signature::

        offspring = cross(random, mom, dad, args)

    This function is most commonly used as a function decorator with
    the following usage::

        @crossover
        def cross(random, mom, dad, args):
            # Implementation of paired crossing

    The generated function also contains an attribute named
    ``single_crossover`` which holds the original crossover function.
    In this way, the original single-candidate function can be
    retrieved if necessary.

    """
    def ecspy_crossover(random, candidates, args):
        cand = list(candidates)
        if len(cand) % 2 == 1:
            cand = cand[:-1]
        moms = cand[::2]
        dads = cand[1::2]
        children = []
        for i, (mom, dad) in enumerate(zip(moms, dads)):
            cross.index = i
            offspring = cross(random, mom, dad, args)
            for o in offspring:
                children.append(o)
        return children
    ecspy_crossover.__name__ = cross.__name__
    ecspy_crossover.__dict__ = cross.__dict__
    ecspy_crossover.__doc__ = cross.__doc__
    ecspy_crossover.single_crossover = cross
    return ecspy_crossover


@crossover
def n_point_crossover(random, mom, dad, args):
    """Return the offspring of n-point crossover on the candidates.

    This function assumes that the candidate solutions are sliceable.
    It selects n random points without replacement at which to 'cut'
    the candidate solutions and recombine them.

    .. Arguments:
       random -- the random number generator object
       mom -- the first parent candidate
       dad -- the second parent candidate
       args -- a dictionary of keyword arguments

    Optional keyword arguments in args:
    
    - *crossover_rate* -- the rate at which crossover is performed 
      (default 1.0)
    - *num_crossover_points* -- the n crossover points used (default 1)
    
    """
    crossover_rate = args.setdefault('crossover_rate', 1.0)
    num_crossover_points = args.setdefault('num_crossover_points', 1)
    children = []
    if random.random() < crossover_rate:
        num_cuts = min(len(mom)-1, num_crossover_points)
        cut_points = random.sample(range(1, len(mom)), num_cuts)
        cut_points.sort()
        bro = copy.copy(dad)
        sis = copy.copy(mom)
        normal = True
        for i, (m, d) in enumerate(zip(mom, dad)):
            if i in cut_points:
                normal = not normal
            if not normal:
                bro[i] = m
                sis[i] = d
        children.append(bro)
        children.append(sis)
    else:
        children.append(mom)
        children.append(dad)
    return children


@crossover
def uniform_crossover(random, mom, dad, args):
    """Return the offspring of uniform crossover on the candidates.

    This function assumes that the candidate solutions are iterable.
    For each element of the parents, a biased coin is flipped to 
    determine whether the first offspring gets the 'mom' or the 
    'dad' element. An optional keyword argument in args, ``pux_bias``, 
    determines the bias.

    .. Arguments:
       random -- the random number generator object
       mom -- the first parent candidate
       dad -- the second parent candidate
       args -- a dictionary of keyword arguments

    Optional keyword arguments in args:
    
    - *crossover_rate* -- the rate at which crossover is performed 
      (default 1.0)
    - *pux_bias* -- the bias toward the first candidate in the crossover 
      (default 0.5)
    
    """
    pux_bias = args.setdefault('pux_bias', 0.5)
    crossover_rate = args.setdefault('crossover_rate', 1.0)
    children = []
    if random.random() < crossover_rate:
        bro = copy.copy(dad)
        sis = copy.copy(mom)
        for i, (m, d) in enumerate(zip(mom, dad)):
            if random.random() < pux_bias:
                bro[i] = m
                sis[i] = d
        children.append(bro)
        children.append(sis)
    else:
        children.append(mom)
        children.append(dad)
    return children


@crossover
def blend_crossover(random, mom, dad, args):
    """Return the offspring of blend crossover on the candidates.

    This function assumes that the candidate solutions are iterable
    and composed of values on which arithmetic operations are defined.
    It performs blend crossover, which is similar to a generalized 
    averaging of the candidate elements. This function also 
    makes use of the bounder function as specified in the EC's 
    ``evolve`` method.

    .. Arguments:
       random -- the random number generator object
       mom -- the first parent candidate
       dad -- the second parent candidate
       args -- a dictionary of keyword arguments

    Optional keyword arguments in args:
    
    - *crossover_rate* -- the rate at which crossover is performed 
      (default 1.0)
    - *blx_alpha* -- the blending rate (default 0.1)
    
    """
    blx_alpha = args.setdefault('blx_alpha', 0.1)
    crossover_rate = args.setdefault('crossover_rate', 1.0)
    bounder = args['_ec'].bounder
    children = []
    if random.random() < crossover_rate:
        bro = copy.copy(dad)
        sis = copy.copy(mom)
        for i, (m, d) in enumerate(zip(mom, dad)):
            smallest = min(m, d)
            largest = max(m, d)
            delta = blx_alpha * (largest - smallest)
            bro[i] = smallest - delta + random.random() * (largest - smallest + 2 * delta)
            sis[i] = smallest - delta + random.random() * (largest - smallest + 2 * delta)
        bro = bounder(bro, args)
        sis = bounder(sis, args)
        children.append(bro)
        children.append(sis)
    else:
        children.append(mom)
        children.append(dad)
    return children
    
    
def differential_crossover(random, candidates, args):
    """Return the offspring of differential crossover on the candidates.

    This function assumes that the candidate solutions are iterable
    and composed of values on which arithmetic operations are defined.
    It performs differential crossover, which is similar to the update
    rule used in particle swarm optimization. This function also 
    makes use of the bounder function as specified in the EC's 
    ``evolve`` method.

    .. Arguments:
       random -- the random number generator object
       candidates -- the candidate solutions
       args -- a dictionary of keyword arguments

    Optional keyword arguments in args:
    
    - *crossover_rate* -- the rate at which crossover is performed 
      (default 1.0)
    - *differential_phi* -- the amount of random change in the crossover 
      (default 0.1)
    
    """
    differential_phi = args.setdefault('differential_phi', 0.1)
    crossover_rate = args.setdefault('crossover_rate', 1.0)
    bounder = args['_ec'].bounder
        
    cand = list(candidates)
    if len(cand) % 2 == 1:
        cand = cand[:-1]
        
    # Since we don't have fitness information in the candidates, we need 
    # to make a dictionary containing the candidate and its corresponding 
    # individual in the population.
    population = args['_ec'].population[:]
    lookup = dict(zip([tuple(p.candidate) for p in population], population))
    
    moms = cand[::2]
    dads = cand[1::2]
    children = []
    for mom, dad in zip(moms, dads):
        if random.random() < crossover_rate:
            bro = copy.copy(dad)
            sis = copy.copy(mom)
            mom_is_better = lookup[tuple(mom)] > lookup[tuple(dad)]
            for i, (m, d) in enumerate(zip(mom, dad)):
                negpos = 1 if mom_is_better else -1
                val = d if mom_is_better else m
                bro[i] = val + differential_phi * random.random() * negpos * (m - d)
                sis[i] = val + differential_phi * random.random() * negpos * (m - d)
            bro = bounder(bro, args)
            sis = bounder(sis, args)
            children.append(bro)
            children.append(sis)
        else:
            children.append(mom)
            children.append(dad)
    return children
    

@crossover
def simulated_binary_crossover(random, mom, dad, args):
    """Return the offspring of simulated binary crossover on the candidates.
    
    This function performs simulated binary crosssover, following the 
    implementation in NSGA-II 
    `(Deb et al., ICANNGA 1999) <http://vision.ucsd.edu/~sagarwal/icannga.pdf>`_.
 
    .. Arguments:
       random -- the random number generator object
       mom -- the first parent candidate
       dad -- the second parent candidate
       args -- a dictionary of keyword arguments

    Optional keyword arguments in args:
    
    - *sbx_etac* -- the non-negative distribution index (default 10)
    
    A small value of the `sbx_etac` optional argument allows solutions 
    far away from parents to be created as children solutions, while a 
    large value restricts only near-parent solutions to be created as
    children solutions.
    
    """
    etac = args.setdefault('sbx_etac', 10)
    bounder = args['_bounder']
    bro = copy.copy(dad)
    sis = copy.copy(mom)
    for i, (m, d, lb, ub) in enumerate(zip(mom, dad, bounder.lower_bound, bounder.upper_bound)):
        try:
            if m > d:
                m, d = d, m
            beta = 1.0 + 2 * min(m - lb, ub - d) / float(d - m)
            alpha = 2.0 - 1.0 / beta**(eta_c + 1.0)
            u = random.random() 
            if u <= (1.0 / alpha):
                beta_q = (u * alpha)**(1.0 / float(eta_c + 1.0))
            else:
                beta_q = (1.0 / (2.0 - u * alpha))**(1.0 / float(eta_c + 1.0))
            bro_val = 0.5 * ((m + d) - beta_q * (d - m))
            bro_val = max(min(bro_val, ub), lb)        
            sis_val = 0.5 * ((m + d) + beta_q * (d - m))
            sis_val = max(min(sis_val, ub), lb)
            if random.random() > 0.5:
                bro_val, sis_val = sis_val, bro_val
            bro[i] = bro_val
            sis[i] = sis_val
        except ZeroDivisionError:
            # The offspring already have legitimate values for every element,
            # so no need to take any special action here.
            pass
    return [bro, sis]


@crossover
def laplace_crossover(random, mom, dad, args):
    a = args.setdefault('laplace_a', 1)
    b = args.setdefault('laplace_b', 0)
    bro = copy.copy(dad)
    sis = copy.copy(mom)
    for i, (m, d) in enumerate(zip(mom, dad)):
        u = random.random()
        if random.random() <= 0.5:
            beta = a - b * math.log(u)
        else:
            beta = a + b * math.log(u)
        bro[i] = m + beta * math.abs(m - d)
        sis[i] = d + beta * math.abs(m - d)
    return [bro, sis]
    










'''
(Self Modifying) Cartesian Genetic Programming. 
implemented as specified in the paper:
"Developments in Cartesian Genetic Programming: self-modifying CGP"
by Harding, Miller & Banzhaf (2010)

(no self modification yet)  


'''
__author__ = 'Henrik Rudstrom'
from docutils.parsers.rst.directives import encoding
import inspect
from ecspy.variators.mutators import mutator

import networkx, random, math, unittest, time
from itertools import izip_longest, dropwhile

class CGPEncoding:
    '''
       Defines a one single row cgp genome encoding.
    '''
    
    
    def __init__(self, length, max_arity, levelsback, user_functions, min_outputs=1):
        '''
        arguments:

        - *length* -- number of columns in the cgp matrix
        - *max_arity* -- tuple (number of edges pr node, number of data fields pr node)
        - *user_functions* -- list of node functions 
        - *levelsback* -- parameter that limits how many columns back a node can connect to, minimum 1
        - *min_outputs* -- minimum outputs needed
        '''
        self.length = length
        self.arity = max_arity
        self.levelsback = levelsback
        self.user_functions = user_functions
        
        self.min_outputs = min_outputs
        
        self.function_names = ['inp', 'inpp', 'out']
        self.function_arity = [0,0,1]
        for f in user_functions:
            self.function_names.append(f.__name__)
            self.function_arity.append(self.get_arity(f))
        self._function_count = len(self.function_names)
        
    def get_arity(self, function):
        args = len(inspect.getargspec(function))
        if hasattr(function, '__self__'):
            return args
        return args-1
        
            
        
    def generator(self):
        '''
        returns a generator function that generates a candidate
        '''
        def random_edges(col):
            return [random.randint(1, self.levelsback) for _ in xrange(self.arity)]
        
        def random_func(col):
            return random.randint(0, self._function_count-1)
        
        def gen(random, args):
            matrix = []
            for col in xrange(self.length):
                node = [random_func(col)]
                node += random_edges(col)
                matrix.append(node)
            return matrix
        return gen

    def mutator(self):        
        def mutate_edge(v):
            return (v + random.randint(1, self.levelsback)) % self.levelsback
        
        def mutate_func(v):
            l = self._function_count
            return (v + random.randint(1, l)) % l 
        
        def mut(random, cand, args):
            maybe_mutate = lambda v, func: func(v) if random.random() < args['mutation_rate'] else v
            matrix = []
            for col in cand:
                node = [maybe_mutate(col[0], mutate_func)] 
                node += [maybe_mutate(c, mutate_edge) for c in col[1:1+self.arity]]
                matrix.append(node)
                
            return matrix
        
        return mutator(mut)

class CGPPhenotype:
    def __init__(self, encoding, candidate):
        self.encoding = encoding
        self.candidate = candidate

    
#    def get_outputs(self):
#        outputs = []
#        for i, c in enumerate(self.candidate):
#            if c[0] == 2:
#                outputs.append(i)
#        return outputs

    def get_inputs(self, node):
        if node == 0:
            return []
        
        return [(node - i) % node for i in self.candidate[node][1:1+self.encoding.arity]]  
        
    def traverse(self, outputs=None):
        '''yields the nodes and their inputs in depth-first ordering'''
        
        nlist = outputs or self.get_outputs()
        seen={} # nodes seen      
        for source in nlist:
            if source in seen: continue
            queue=[source]     # use as LIFO queue
            while queue:
                v=queue[-1]
                if v not in seen:
                    seen[v]=True
                done=1
                inputs = self.get_inputs(v)
                for w in inputs:
                    if w not in seen:
                        queue.append(w)
                        done=0
                        break
                if done == 1:
                    yield v, inputs
                    queue.pop()



    def get_outputs(self):
        outputs = []
        for i, c in enumerate(self.candidate):
            if c[0] == 2 and i != 0: #first column cant be output
                outputs.append(self.get_inputs(i)[0])
        
        '''
        A number of measures need to be taken when the number of OUTPUT nodes 
        does not match the number of program outputs.
        - If there are no OUTPUT nodes in the graph, then the last n nodes in the graph are used
        - If there are more OUTPUT nodes than are required, then the right-most OUTPUT 
            nodes are used until the required number of outputs is reached.
        - If the graph has fewer OUTPUT nodes than are required graph, then nodes are 
        chosen as outputs by moving forwards from the right-most node flagged as an output.
        - If there is a condition where not enough nodes can be used as outputs (as there are 
        not enough nodes in the graph), the individual is labeled as corrupt and is given a 
        bad fitness score to prevent selection.
        '''
        #print "outputs", outputs
        mino = self.encoding.min_outputs
        if len(outputs) == 0:
            outputs = [len(self.candidate)-mino+i for i in xrange(mino)]
        while len(outputs) < self.encoding.min_outputs:
            outputs.append(outputs[-1]+1)
        return outputs
    
    def __call__(self, inputs, alternate_functions=None):
        io = FunctionIO(inputs)
        functions = [io.inp, io.inpp, io.out] + (alternate_functions or self.encoding.user_functions)
        
        values = {}
        
        for index, inp in self.traverse():
            
            node = self.candidate[index]
            #print index, inp, node
            f = node[0]
            if index == 0: #first node can only be input node
                f = f % 2
            
            args = [values[i] for i in inp[0:self.encoding.function_arity[f]]]
            val = functions[f](*args)
            values[index] = val
        
        return [values[inp] for inp in self.get_outputs()]
    
    def __repr__(self):
        values = {}
        for index, inp in self.traverse():
            node = self.candidate[index]
            name = self.encoding.function_names[node[0]]
            args = [values[i] for i in inp[0:self.encoding.function_arity[node[0]]]]
            values[index] = "%s(%s)" % (name, ", ".join(args))
        return "\n".join(["f(%s): %s" % (i, values[i]) for i in self.get_outputs()])
            

                


class FunctionIO:
    '''
    INP, INPP and SKIPINP. When decoding the phenotype graph, a pointer is maintained that refers to an input. 
    If the first occurrence of an active input function is INP it returns the first program input. If the first 
    occurrence is INPP, the last program input is returned. After INP is called it increments the input pointer. 
    INPP decrements the input pointer.
    '''
    def __init__(self, inputs):
        self.inputs = inputs
        self.pointer = None
        self.n_inputs = len(self.inputs)
       
    def inp(self):
        if self.pointer is None:
            self.pointer = 0
        else:
            self.pointer = (self.pointer + 1) % self.n_inputs
        return self.inputs[self.pointer] 
    
    def inpp(self):
        if self.pointer is None:
            self.pointer = len(self.inputs) - 1
        else:
            self.pointer = (self.pointer - 1) % self.n_inputs
        return self.inputs[self.pointer] 
    
    def out(self, val):
        '''output function, passes first argument on, in case it is referenced'''
        return val
                

def create_graph(encoding, candidate):
    graph = networkx.DiGraph()
    for node, inputs in encoding.traverse(candidate):
        if node is None:
            node="out"
            graph.add_node(node)
        else:
            graph.add_node(node, {"f":candidate[node][0]})
        graph.add_edges_from([(i, node) for i in inputs])
    return graph       

def draw_graph(graph, scale = 20):

    def view_pos(col, scale):
        x = col * scale + scale / 5.
        return x, 0

    pos = dict(((n, view_pos(n, 20)) for n in graph.nodes()))
    networkx.draw(graph, pos)

class CGPTest(unittest.TestCase):
    def test_input_output(self):
        enc = CGPEncoding(2, 1, 1, [], 2)
        gen = [[0,0],[0,0], [2, 2], [2,2], [2,3]]
        input = [5,2]
        func = CGPPhenotype(enc, gen)
        res = func(input)
        assert res[0] == input[0]
        assert res[1] == input[1]
        assert res[2] == input[1]
        
    def test_no_outputs(self):
        enc = CGPEncoding(2, 1, 1, [], 2)
        gen = [[0,0],[0,0]]
        input = [5,2]
        func = CGPPhenotype(enc, gen)  

        res = func(input)
        
        assert res[0] == input[0], "res"+res[0]
        assert res[1] == input[1], "res"+res[1]

    def test_too_few_outputs(self):
        enc = CGPEncoding(2, 1, 1, [], 2)
        gen = [[0,0],[0,0], [2,2]]
        input = [5,2]
        func = CGPPhenotype(enc, gen)    
        res = func(input)
        
        assert res[0] == input[0]
        assert res[1] == input[1]  
        
        
    def test_arithmetics(self):
        def add(a, b):
            
            return a + b
        def sub(a,b):
            return a - b
        
        enc = CGPEncoding(2, 2, 1, [add, sub], 1)
        gen1 = [[0,0,0],[0,0,0], [3,2,1], [3,2,2], [4, 2, 1], [2,1,0]]
        func = CGPPhenotype(enc, gen1)    
        input = [1,10]
        #(1 + 10) - (10 + 10) 
        res = func(input)
        assert res[0] == -9





from ecspy import ec
from ecspy import terminators

class MicroEC(ec.EvolutionaryComputation):
    def __init__(self, random):
        ec.EvolutionaryComputation.__init__(self, random)
        
    def evolve(self, generator, evaluator, pop_size=10, seeds=[], maximize=True, bounder=ec.bounder(), **args):
        self._kwargs = args
        self._kwargs['_ec'] = self
        self.termination_cause = None
        self.generator = generator
        self.evaluator = evaluator
        self.bounder = bounder
        self.maximize = maximize
        self.population = []
        self.archive = []
        self.num_generations = 0
        self.num_evaluations = 0
        microseeds = seeds
        args.setdefault('min_diversity', 0.05)
        while not self._should_terminate(self.population, self.num_generations, self.num_evaluations):
            microec = ec.EvolutionaryComputation(self._random)
            microec.selector = self.selector
            microec.variator = self.variator
            microec.replacer = self.replacer
            microec.terminator = terminators.diversity_termination
            result = microec.evolve(generator=generator, evaluator=evaluator, pop_size=pop_size, seeds=microseeds, maximize=maximize, **args)
            microseeds = [result[0].candidate]
            self.population = list(result)
            self.num_evaluations += microec.num_evaluations

            # Migrate individuals.
            self.population = self.migrator(random=self._random, population=self.population, args=self._kwargs)
            
            # Archive individuals.
            pop_copy = list(self.population)
            arc_copy = list(self.archive)
            self.archive = self.archiver(random=self._random, archive=arc_copy, population=pop_copy, args=self._kwargs)
            
            self.num_generations += microec.num_generations
            if isinstance(self.observer, (list, tuple)):
                for obs in self.observer:
                    obs(population=self.population, num_generations=self.num_generations, num_evaluations=self.num_evaluations, args=self._kwargs)
            else:
                self.observer(population=self.population, num_generations=self.num_generations, num_evaluations=self.num_evaluations, args=self._kwargs)
        return self.population
    
    

    
       
if __name__ == '__main__':
    import random
    import math
    import time
    from ecspy import ec
    from ecspy import observers
    from ecspy import terminators
    from ecspy import selectors
    from ecspy import replacers
    from ecspy import variators
    from ecspy import archivers


    def rastrigin_generator(random, args):
        return [random.uniform(-5.12, 5.12) for _ in range(2)]

    def rastrigin_evaluator(candidates, args):
        fitness = []
        for cand in candidates:
            fitness.append(10 * len(cand) + sum([x**2 - 10 * (math.cos(2*math.pi*x)) for x in cand]))
        return fitness
        
    prng = random.Random()
    prng.seed(time.time())
    micro = MicroEC(prng)
    micro.selector = selectors.tournament_selection
    micro.replacer = replacers.steady_state_replacement
    micro.variator = [variators.uniform_crossover, variators.gaussian_mutation]
    micro.archiver = archivers.best_archiver
    micro.observer = observers.screen_observer
    micro.terminator = terminators.evaluation_termination
    final_pop = micro.evolve(rastrigin_generator, rastrigin_evaluator, pop_size=10, maximize=False, bounder=ec.bounder(-5.12, 5.12),
                             max_evaluations=3000, num_selected=2, stdev=0.1)
                             
    print('Actual evaluations: %d' % micro.num_evaluations)

    for p in micro.archive:
        print p
